from __future__ import annotations

from tcsoa.gen.BusinessObjects import ChangeItemRevision, ChangeItem, PSBOMViewRevision, BusinessObject, BOMEdit, BOMSupersedure, BOMWindow
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateChangeItemInputs(TcBaseObj):
    """
    Input structure for holding create change item input.
    
    :var clientID: Tracking client invoking this method this information not persisted, Optional
    :var iD: iD
    :var revisionID: The revision ID that will be set on the new object, Required
    :var name: Name that will be used on the new object, Required
    :var description: Description that will be used on the new object, Required
    :var type: This will be used to determine what type of change item that is to be created, Required
    :var relationshipData: The list of objects and their relations to the change item, Optional
    :var extendedData: The list of additional attributes that might be passed from a client, Optional
    """
    clientID: str = ''
    iD: str = ''
    revisionID: str = ''
    name: str = ''
    description: str = ''
    type: str = ''
    relationshipData: List[RelationshipData] = ()
    extendedData: List[ExtendedAttributes] = ()


@dataclass
class CreateChangeItemsOutput(TcBaseObj):
    """
    Output structure for createChangeItems operation
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var changeItem: Change item object reference that was created.
    :var itemRev: change item revision that is created.
    """
    clientId: str = ''
    changeItem: ChangeItem = None
    itemRev: ChangeItemRevision = None


@dataclass
class CreateChangeItemsResponse(TcBaseObj):
    """
    Response structure for createItems operation
    
    :var output: A list of CreateChangeItemsOutput
    :var serviceData: Standard ServiceData member
    """
    output: List[CreateChangeItemsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateSupercedureInput(TcBaseObj):
    """
    Input structure for createSupercedures operation
    
    :var clientID: A unique client identifier. This is a unique string supplied by the caller. This ID 
    is used to identify return data elements and partial errors associated with this input structure.
    
    :var solutionBvr: This parameter represents a reference to the solution item BOM View Revision 
    (BVR) and is a required parameter. A solution item is a new component that replaces an old component in the parent
    assembly. Solution items are supplementary components that are involved in the solution the change is bringing
    about and the affected item represents the final assembly product containing the solution items.
    
    :var isTransferred: This required parameter determines if the Supercedure created is a transfer type 
    Supercedure and takes in a boolean true or false value. Transfer Supercedures are generated when a component is
    moved from one assembly to another.
    
    :var bomAdds: A list of BOM Adds which represents the additions made to the assembly. It is a 
    required input element.
    
    :var bomCancels: A list of BOM Cancels which denotes the deletions made from the assembly and 
    is a required input parameter.
    
    :var extendedAttributes: A placeholder for additional attributes and this is an optional parameter. i.e it can be
    empty.
    """
    clientID: str = ''
    solutionBvr: PSBOMViewRevision = None
    isTransferred: bool = False
    bomAdds: List[BusinessObject] = ()
    bomCancels: List[BusinessObject] = ()
    extendedAttributes: ExtendedAttributes = None


@dataclass
class CreateSupercedureOutput(TcBaseObj):
    """
    Output structure for createSupercedures operation
    
    :var clientId: Identifier that helps the client to track the object(s) created.
    :var supercedure: List of tags to the supercedure reference that was created
    """
    clientId: str = ''
    supercedure: List[BusinessObject] = ()


@dataclass
class CreateSupercedureResponse(TcBaseObj):
    """
    Response structure containing list of Supercedure objects and partial errors.
    
    :var output: This represents a list of created Supercedure objects.
    :var serviceData: The SOA framework object containing Supercedure objects that were created 
    based on the input and error information returned based on validations performed.
    """
    output: List[CreateSupercedureOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ConfigureChangeSearchData(TcBaseObj):
    """
    'ConfigureChangeSearchData' structure represents all the details of a Change Home folder.
    
    :var name: Name of the folder
    :var visible: Indicates if the folder is visible or not visible.
    :var queryName: Indicates the name of the query.
    :var savedSearchName: Indicates the name of the saved search name.
    :var queryCriteria: Indicates saved query criterias.
    :var queryValues: Indicates saved query values.
    :var isFolderSiteLevel: Indicates folder is site level or user level and default is user level which is false.
    :var operation: Indicates folder to be added or removed to add value should be Add and to delete value should be
    Remove.
    """
    name: str = ''
    visible: bool = False
    queryName: str = ''
    savedSearchName: str = ''
    queryCriteria: List[str] = ()
    queryValues: List[str] = ()
    isFolderSiteLevel: bool = False
    operation: str = ''


@dataclass
class ExtendedAttributes(TcBaseObj):
    """
    Input structure for createItems operation to support setting attribute values on the created Item,
    ItemRevision, or corresponding master forms that may be created with the objects.
    
    :var attributeName: The name of the attribte to modify or change or update.
    :var values: Values for the attribute ateast must have one value.
    """
    attributeName: str = ''
    values: List[str] = ()


@dataclass
class FindContextDataInputs(TcBaseObj):
    """
    'FindContextDataInputs' structure contains an object reference to a change revision, a pair of object references to
    any Business Object type, and a context string which determines what and how the input objects will be used to
    obtain the desired output objects.
    
    :var changeRev: An object reference to a change revision.
    :var primaryContextInputData: An object reference to any BO type.
    :var secondaryContextInputData: An object reference to any BO type.
    :var contextRelName: A context string.
    """
    changeRev: ChangeItemRevision = None
    primaryContextInputData: BusinessObject = None
    secondaryContextInputData: BusinessObject = None
    contextRelName: str = ''


@dataclass
class FindContextDataOutput(TcBaseObj):
    """
    'FindContextDataOutput' structure contains an object reference that can be used to point to a change revision, a
    list that can be used to hold any number of returned business objects, and a list of counts to help sort out the
    Adds and the Cancels from the returned object list.
    
    :var changeRev: A reference to change revision. Can be empty.
    :var contextOutputData: A list to hold any number of returned business objects.
    :var vBomEditCount: A list of counts used to sort out the Adds and the Cancels.
    """
    changeRev: ChangeItemRevision = None
    contextOutputData: List[BusinessObject] = ()
    vBomEditCount: List[int] = ()


@dataclass
class FindContextDataResponse(TcBaseObj):
    """
    'FindContextDataResponse' structure contains a list of 'FindContextDataOutput' structures and a service data.
    
    :var output: A list of 'FindContextDataOutput' structures.
    :var serviceData: A service data.
    """
    output: List[FindContextDataOutput] = ()
    serviceData: ServiceData = None


@dataclass
class FindSupersedureOutput(TcBaseObj):
    """
    Output structure find supercedure.
    
    :var findsupercedure: A list of reference to supersedure.
    :var bomedit: A reference to BOMEdit to which a supersedure is associated to.
    """
    findsupercedure: List[BOMSupersedure] = ()
    bomedit: BOMEdit = None


@dataclass
class FindSupersedureResponse(TcBaseObj):
    """
    Output structure for list of BOMSupersedure and ServiceData.
    
    :var supersedureOutput: A reference to list of supersedures.
    :var serviceData: The SOA framework object containing Supercedure objects that were 
    returned based on the input and error information returned based on validations performed.
    """
    supersedureOutput: List[FindSupersedureOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetAllChangeHomeFolderResponse(TcBaseObj):
    """
    'GetAllChangeHomeFolderResponse' structure encapsulates the list of returned Change Home folders as a vector of
    'ConfigureChangeSearchData' structures, in addition to the standard mandatory 'ServiceData' structure.
    
    :var output: A vector of 'ConfigureChangeSearchData' structures which represent the details of change home folders
    :var serviceData: Standard ServiceData member
    """
    output: List[ConfigureChangeSearchData] = ()
    serviceData: ServiceData = None


@dataclass
class GetBOMEditInput(TcBaseObj):
    """
    'GetBOMEditInput' structure represents a set of criteria for finding any existing BOMEdit object(s) that satisfy
    the criteria.  It is possible that none satisfies the input criteria.
    
    :var changeRev: A reference to the revision of the change item, Required.
    :var affectedBvr: Reference to PSBOMViewRevision of the affected item, Required
    :var bomEditType: The type of this BOMEdit valid integer values are 1 to 10. BOM_Add=1,BOM_Cancel=2,
    BOM_Quantity_Change=3, BOM_Move=4,BOM_Reshape=5,BOM_Note_Change=6,BOM_Variant_Change=7,LBOM_Add=8,LBOM_Cancel=9,
    and LBOM_Quantity_Change=10
    """
    changeRev: ChangeItemRevision = None
    affectedBvr: PSBOMViewRevision = None
    bomEditType: int = 0


@dataclass
class GetBOMEditOutput(TcBaseObj):
    """
    GetBOMEditOutput structure contains a list of BOMEdit objects and the associated change revision object. The list
    contains the BOMEdit objects that satisfy the input criteria as represented by the corresponding GetBOMEditInput
    structure, and it can be empty.
    
    :var bomEdits: The BOMEdit objects that satisfy the input criteria as represented by the corresponding
    GetBOMEditInput structure. This can be empty.
    :var changeRev: The change revision associated with the returned BOMEdit object(s).
    """
    bomEdits: List[BOMEdit] = ()
    changeRev: ChangeItemRevision = None


@dataclass
class GetBOMEditResponse(TcBaseObj):
    """
    'GetBOMEditResponse' structure contains a list of 'GetBOMEditOutput' structures and a standard ServiceData.
    
    :var bomEditOutput: A vector of 'GetBOMEditOutput' structures.  See 'GetBOMEditOutput' section below for more
    details.  This can be empty.
    :var serviceData: Standard ServiceData member
    """
    bomEditOutput: List[GetBOMEditOutput] = ()
    serviceData: ServiceData = None


@dataclass
class RelationshipData(TcBaseObj):
    """
    Input structure for holding relationship data.
    
    :var tags: List of object references used to hold the objects to put in the relationship, Required
    :var relTypeName: Name of relationship to be created, Required
    :var operation: Flag indicating weather to add or remove relationship, Required
    """
    tags: List[BusinessObject] = ()
    relTypeName: str = ''
    operation: str = ''


@dataclass
class UpdateChangeProperties(TcBaseObj):
    """
    Input structure for updateChangeItems operation
    
    :var chgRev: A reference to the revision of the change item  that to be updated, Required
    :var newPropertyValues: A vector of the property values, Optional
    :var newRelationshipData: A vector of  of relationship data , Optional
    """
    chgRev: ChangeItemRevision = None
    newPropertyValues: List[ExtendedAttributes] = ()
    newRelationshipData: List[RelationshipData] = ()


@dataclass
class UpdateSupercedureData(TcBaseObj):
    """
    Structure for 'updateSupercedures' operation.
    
    :var bomAddOrCancleFlag: A flag value indicating if the update operation is to be carried out on 
    BOMAdds or BOMCancels. This parameter uses Change Management BOM_ADD or BOM_CANCEL constants.
    
    :var operation: A flag indicating whether to add or remove components from the 
    Supercedure. The parameter uses Change Management constants OPERARTION_TYPE_ADD or OPERATION_TYPE_CANCEL.
    
    :var tags: A list of tag values to be added or removed.
    :var supTag: This input element represents the tag for the Supercedure to be updated.
    """
    bomAddOrCancleFlag: str = ''
    operation: str = ''
    tags: List[BusinessObject] = ()
    supTag: BusinessObject = None


@dataclass
class CreateBOMEditInput(TcBaseObj):
    """
    'CreateBOMEditInput' structure contains the object reference to the associated change revision, and the object
    references to the two BOM windows.  The referenced objects will provide the necessary details to create a number of
    BOMEdit objects that represents the whole BOM Change.
    
    :var changeRev: A reference to the revision of the change item.
    :var firstWindow: A reference to the BOM window containing the BOM view Revision of Solution Item Revision for the
    given change objects.
    :var secondWindow: A reference to the BOM window containing the BOM view Revision of Impacted Item Revision for the
    given change objects.
    """
    changeRev: ChangeItemRevision = None
    firstWindow: BOMWindow = None
    secondWindow: BOMWindow = None


@dataclass
class CreateBOMEditResponse(TcBaseObj):
    """
    CreateBOMEditResponse structure contains an empty output, and a standard ServiceData.
    
    :var output: A reference to list of BOMEdits.
    :var serviceData: Standard ServiceData member
    """
    output: List[BOMEdit] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, VariantRule, RevisionRule, Awb0ProductContextInfo
from tcsoa.gen.Internal.ActiveWorkspaceBom._2017_06.OccurrenceConfiguration import EffectivityInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class OptionValuesCursor(TcBaseObj):
    """
    OptionValuesCursor is a cursor that is returned for use in subsequent calls to get next page of results.
    
    :var startReached: If true, the first page of the results has been reached.
    :var endReached: If true, the last page of the results has been reached.
    :var startIndex: The Cursor start position for the result returned so far.
    :var endIndex: The Cursor end position for the result returned so far.
    :var pageSize: The maximum number of results that can be returned in one service call.
    :var isForward: If true scrolling is done in forward direction.
    """
    startReached: bool = False
    endReached: bool = False
    startIndex: int = 0
    endIndex: int = 0
    pageSize: int = 0
    isForward: bool = False


@dataclass
class RevisionRuleInfo(TcBaseObj):
    """
    'RevisionRuleInfo' contains list of RevisionRule objects along with their serialized strings.
    
    :var revisionRule: The RevisionRule.
    :var serializedRevRule: A string representing transient RevisionRule.This string is system generated which holds an
    information about name , description and all clauses in the RevisionRule as per the precedence order.Format of the
    string is encoded and it is for internal use only.
    """
    revisionRule: RevisionRule = None
    serializedRevRule: str = ''


@dataclass
class VariantContent(TcBaseObj):
    """
    Contains variant rule, the saved variant rules associated with the window, list of option value details. This
    structure is used for both getClassicVariants as well as createOrUpdateClassicVariantRule.
    
    :var variantRule: A VariantRule with which the current product is configured. This can be NULL
    :var variantOptionValueEntry: A list of options associated with VariantRule.
    """
    variantRule: VariantRule = None
    variantOptionValueEntry: List[VariantOptionValueEntry] = ()


@dataclass
class VariantOptionFilterData(TcBaseObj):
    """
    Option details for which option values are to be returned.
    
    :var optionUID: UID of the Variant that contains details of options and values.
    :var owningItemUID: UID of the Item on which option is set.
    """
    optionUID: str = ''
    owningItemUID: str = ''


@dataclass
class VariantOptionValue(TcBaseObj):
    """
    Contains options and configuration details.
    
    :var optionValue: Value assigned to an option.
    :var howset: It indicates how value is set. Valid values are as follows. 
    0: value is unset, 
    4: value is set by user.
    """
    optionValue: str = ''
    howset: int = 0


@dataclass
class VariantOptionValueEntry(TcBaseObj):
    """
    Options associated with VariantRule
    
    :var optionUID: UID of the Variant that contains details of options and values.
    :var owningItemUID: UID of the Item on which option is set.
    :var optionName: Name of the option associated.
    :var optionDesc: Description of the option.
    :var variantOptionValue: A list containing options and configuration details.
    """
    optionUID: str = ''
    owningItemUID: str = ''
    optionName: str = ''
    optionDesc: str = ''
    variantOptionValue: List[VariantOptionValue] = ()


@dataclass
class ClassicVariantsData(TcBaseObj):
    """
    Input for variant data request.
    
    :var productContext: Awb0ProductContextInfo of the product for which Variant information to be retrieved.
    :var variantRule: VariantRule for which option and value information are to be retrieved. This input is optional.
    If provided in the input, the option values returned will have a flag indicating option values set in the
    VariantRule.
    :var optionFilter: Option details for which option values are to be returned. An empty string value in this input
    element will return all options and their associated values.
    :var optionValuesCursor: A cursor that is returned for use in subsequent calls to get next page of results.
    """
    productContext: Awb0ProductContextInfo = None
    variantRule: VariantRule = None
    optionFilter: VariantOptionFilterData = None
    optionValuesCursor: OptionValuesCursor = None


@dataclass
class ClassicVariantsResp(TcBaseObj):
    """
    List of options and their values.
    
    :var productContext: Awb0ProductContextInfo for the product for which Variant information is retrieved.
    :var variantContent: Options and values associated with the product.
    :var serviceData: The service data containing updated object(s).
    """
    productContext: Awb0ProductContextInfo = None
    variantContent: VariantContent = None
    serviceData: ServiceData = None


@dataclass
class ConfigRuleResponse2(TcBaseObj):
    """
    ConfigRuleResponse2 contains RevisionRule and VariantRule information.
    
    :var marker: Number of preferred RevisionRule or VariantRule. Preferred RevisionRule or VariantRule are the one
    which are indexed through structure indexing. The default value is -1.
    :var endIndex: End index of page.
    :var totalFound: Total numbers of RevisionRule or VariantRule found.
    :var globalRevisionRule: Global RevisionRule based on preference "TC_config_rule_name".
    :var revisionRules: List of unique RevisionRule objects along with serialized string.
    :var preferredEffectivityInfo: Fetch the preferred effectivity information based on prdouct and RevisionRule only
    when 'fetchOnlyPreferredConfiguration' is true.
    :var addOpenObjAsPreferredEndItem: If true, none of the preferred effectivity end item 
    revision  or preferred VariantRule owninig object is opened product and the client will use the opened product as
    preferred end item or VariantRule owninig object .
    :var variantRules: List of VariantRule based on RevisionRule and owning object for VariantRule.
    :var preferredVarRuleOwningObjects: Fetch the list of owning object based on RevisionRule and product only when
    'fetchOnlyPreferredConfiguration' is true.
    :var serviceData: The Service Data.
    """
    marker: int = 0
    endIndex: int = 0
    totalFound: int = 0
    globalRevisionRule: RevisionRule = None
    revisionRules: List[RevisionRuleInfo] = ()
    preferredEffectivityInfo: EffectivityInfo = None
    addOpenObjAsPreferredEndItem: bool = False
    variantRules: List[VariantRule] = ()
    preferredVarRuleOwningObjects: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class CreateOrUpdateClassicVariantRuleData(TcBaseObj):
    """
    Input containing Awb0ProductContextInfo, user selected options and values and an optional saved VariantRule.
    
    :var productContext: Awb0ProductContextInfo of the product for which option and values are to be set.
    :var variantRuleName: Name of the VariantRule to be created.
    :var variantContent: A list of options and values provided by the user with an optional VariantRule. The input
    options and values are applied to the currently opened product.
    :var saveRule: If true, the input VariantRule will be updated. If false, input VariantRule will not be updated.
    Access checks will be applied during updation of the VariantRule.
    """
    productContext: Awb0ProductContextInfo = None
    variantRuleName: str = ''
    variantContent: VariantContent = None
    saveRule: bool = False



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Role, User, Person, GroupMember, Group
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GroupMembershipInput(TcBaseObj):
    """
    A structure containing input to retrieve group membership information.
    
    :var searchString: The string to search for within the group membership.
    :var startIndex: The index from which to start the search.
    :var maxToReturn: The maximum number of objects to return.
    :var maxToLoad: The maximum number of objects to load.
    """
    searchString: str = ''
    startIndex: int = 0
    maxToReturn: int = 0
    maxToLoad: int = 0


@dataclass
class GroupMembershipOutput(TcBaseObj):
    """
    Structure that contains a list of groupMember objects with same group and role
    
    :var group: Group object
    :var role: Role object
    :var groupMembers: Group member objects
    """
    group: Group = None
    role: Role = None
    groupMembers: List[GroupMember] = ()


@dataclass
class GroupMembershipResponse(TcBaseObj):
    """
    A service response structure containing group membership information.
    
    :var groupMembers: The list which contains a list of GroupMember business objects with same group and role, the
    outer list is sorted first by group name, then by role name, the inner list is sorted by user name
    :var userToPersonMap: The map that contains a mapping from a User business object to a Person business object.
    :var totalFound: The total number of GroupMember business objects found.
    :var totalLoaded: The total number of GroupMember business objects loaded.
    :var serviceData: The service data object.
    """
    groupMembers: List[GroupMembershipOutput] = ()
    userToPersonMap: UserToPersonMap = None
    totalFound: int = 0
    totalLoaded: int = 0
    serviceData: ServiceData = None


"""
A map that contains a mapping from a User business object to a Person business object.
"""
UserToPersonMap = Dict[User, Person]



from __future__ import annotations

from tcsoa.gen.GlobalMultiSite._2010_04.ImportExport import FileTicket
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ImportObjectsFromPLMXMLWithDSMResp(TcBaseObj):
    """
    The response for importObjectsFromPLMXMLWithDSM operation. It holds the file read tickets for uploading the Dataset
    named reference files, the file read ticket for the import log file, and any partial failures.
    
    :var namedRefPLMDFileTickets: The FMS read tickets are used to upload the Dataset named reference files. These
    files are in .plmd format. On Teamcenster services client, files must be uploaded using these tickets thru Data
    Share Manager system after the calling of this operation. The API DownloadFilesWithDM from IFileManager class can
    be used to perform the upload.
    :var logFileTicket: The FMS read ticket is used to download the generated import log file from the FMS transient
    volume. The API DownLoadTransientFile from FmsFileCacheProxy can be used to perform the download.
    :var serviceData: The service data contains the errors.
    """
    namedRefPLMDFileTickets: List[str] = ()
    logFileTicket: FileTicket = None
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Notification._2014_10.SubscriptionManagement import ActionHandler, AttributeCriteria, NotificationDefinition
from tcsoa.gen.BusinessObjects import ImanRelation, User, TaskType, Condition, ImanActionHandler, BusinessObject, ImanEventType, ImanSubscription
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ExecutionPeriod(TcBaseObj):
    """
    Structure to hold the execution time, frequency and the execution day of the subscription.
    
    :var executionTime: The time of day when the Subscription should be processed. Setting NULL will cause the handlers
    to be run as soon as resources are available.
    :var executionDay: Execution day for the subscription to be created (Sunday, Monday, Tuesday, Wednesday, Thursday,
    Friday, Saturday). 
    :var frequency: Frequency of the subscription to be created (Immediately, Daily, Weekly).
    """
    executionTime: datetime = None
    executionDay: str = ''
    frequency: str = ''


@dataclass
class FindSubscriptionsInput2(TcBaseObj):
    """
    Contains all the input criteria required to search for Subscriptions.
    
    :var target: The object to monitor for the specified event type. This can also be Business Object type for class
    based ImanSubscription. If the value is NULL, search will not be performed on the basis of this field.
    :var subscriber: The User that is subscribing to the event. If the value is NULL, search will not be performed on
    the basis of this field.
    :var isActive: Status of the Subscription object. Valid values are 0, 1 and 2. Default value is 0 which indicates
    active as well as inactive subscription will be searched. 1 indicates active subscriptions and 2 indicates inactive
    subscriptions. Any value other than the valid values will be reinitialized to 0.
    :var frequency: Frequency of the subscription. Valid values are Immediately, Daily and Weekly.
    :var executionDay: Exedution Day of the subscription. Valid values are Sunday, Monday, Tuesday, Wednesday,
    Thursday, Friday or Saturday. If the frequency is Immediately or Daily, then this value may be NULL.
    :var notifier: The User to whom the subscription is transferred. If the value is NULL, search will not be performed
    on the basis of this field.
    :var temporaryNotifierDateRange: From and To date in case of temporary transfer of the subscription. If the value
    is NULLDATE, search will not be performed on the basis of this field.
    
    :var eventType: Type of event that will trigger the subscription. If the value is NULL, search will not be
    performed on the basis of this field.
    :var handlers: The handlers to execute when processing the Subscription. May be an empty list.
    :var condition: The Condition to evaluate when processing this Subscription. Only valid for class based
    Subscriptions. If the value is NULL, search will not be performed on the basis of this field.
    :var name: Name of the subscription object. If the value is NULL, search will not be performed on the basis of this
    field.
    :var executionTimeRange: Upper limit and the lower limit of the Subscription execution time.
    :var expirationDateRange: Upper limit and the lower limit of the Subscription expiration date.
    :var noExpDate: If true, do not search for Subscriptions with an expiration date.
    :var notificationPriority: Notification priority. Valid values are High, Low and Normal.
    """
    target: BusinessObject = None
    subscriber: User = None
    isActive: int = 0
    frequency: str = ''
    executionDay: str = ''
    notifier: User = None
    temporaryNotifierDateRange: DateRange = None
    eventType: ImanEventType = None
    handlers: List[ActionHandler] = ()
    condition: Condition = None
    name: str = ''
    executionTimeRange: DateRange = None
    expirationDateRange: DateRange = None
    noExpDate: bool = False
    notificationPriority: str = ''


@dataclass
class GetAssignEventDataResponse(TcBaseObj):
    """
    The GetAssignEventDataResponse structure represents the output response returning list of TaskType objects with
    partial errors wrapped in serviceData, if any.
    
    :var releaseStatus: A list of release status types.
    :var serviceData: Partial errors will be returned and wrapped in serviceData, if any are specified.
    """
    releaseStatus: List[TaskType] = ()
    serviceData: ServiceData = None


@dataclass
class GetAttachEventDataResponse(TcBaseObj):
    """
    The GetAttachEventDataResponse structure represents the output response returning lists of ImanRelation objects and
    attachment types.  Partial errors will be returned and wrapped in serviceData, if any are specified. 
    
    :var relations: A list of ImanRelation objects.
    :var attachmentTypes: A list of attachment type business objects.
    :var serviceData: Partial errors will be returned and wrapped in serviceData, if any are specified.
    """
    relations: List[ImanRelation] = ()
    attachmentTypes: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class GetEventTypesResponse(TcBaseObj):
    """
    The GetEventTypesResponse structure represents the output response returning a client ID string and lists of
    auditableEventTypes and subscribableEventTypes with partial errors wrapped in serviceData, if any.
    
    :var clientId: The unmodified value from the EventObject.clientId. This can be used by the caller to indentify this
    data structure with the source input data.
    :var auditableEventTypes: A list of auditable ImanEventType elements.
    :var subscribableEventTypes: A list of subscribable ImanEventType elements.
    :var serviceData: Error encountered while processing post event on element in the set is reported as partial errors
    and processing continues for the remaining elements in the input set. 
    """
    clientId: str = ''
    auditableEventTypes: List[ImanEventType] = ()
    subscribableEventTypes: List[ImanEventType] = ()
    serviceData: ServiceData = None


@dataclass
class GetSubscribableTypesResponse(TcBaseObj):
    """
    Contains type objects. Partial errors are returned via serviceData if there are any. 
    
    :var typeInfoList: Business object types.
    :var serviceData: Teamcenter service response data.
    """
    typeInfoList: List[SubscribableTypeInfo] = ()
    serviceData: ServiceData = None


@dataclass
class GetSubscriptionInputResponse(TcBaseObj):
    """
    A structure with lists of ImanEventType objects, ImanActionHandler objects and the attribute criteria specified in
    the configuration file for a business object type. Service data contains partial errors if any.
    
    :var eventTypes: A list of ImanEventType objects.
    :var handlers: A list of ImanActionHandler objects.
    :var attributeCriteria: A list of AttributeCriteria values corresponding to an object type.
    :var serviceData: Partial failures will be returned in the ServiceDate for each failed processing. Error
    encountered while processing post event on element in the set is reported as partial errors and processing
    continues for the remaining elements in the input set.
    """
    eventTypes: List[ImanEventType] = ()
    handlers: List[ImanActionHandler] = ()
    attributeCriteria: List[AttributeCriteria] = ()
    serviceData: ServiceData = None


@dataclass
class ModifySubscriptionsInput2(TcBaseObj):
    """
    Contains the Subscription object to modify and all of the input values to set on the Subscription. Supports both
    object and class based Subscriptions.
    
    :var subscriptionObject: ImanSubscription object to be modified.
    :var newSubscriptionValues: Contains all the input criteria required to create a single object or class based
    Subscription. The inputs include the target, subscriber, eventType, handlers, condition, criteria, revisionOption,
    executionPeriod, expirationDate, notification, notificationPriority, name, isActive.
    """
    subscriptionObject: ImanSubscription = None
    newSubscriptionValues: SubscriptionInput2 = None


@dataclass
class SubscribableTypeInfo(TcBaseObj):
    """
    The structure holds the information of subscribable types that need to be returned in the
    GetSubscribableTypesResponse structure.
    
    :var type: The subscribable business object type.
    :var typeName: The name of the subscribable business object type.
    :var parents: The names of all the parents in the heirarchy of the subscribable business object types.
    """
    type: BusinessObject = None
    typeName: str = ''
    parents: List[str] = ()


@dataclass
class SubscriptionInput2(TcBaseObj):
    """
    Contains all the input criteria required to create a single object or class based Subscription. The main inputs
    include the target object, subscriber, event type and action handler list.
    
    The notification should be included if the IMAN_Smtp_Mail_Notify handler is used.
    
    
    :var target: The object to monitor for the specified event type. This can also be Business Object type for class
    based ImanSubscription.
    :var subscriber: The User that is subscribing to the event.
    :var notification: Specifies the parameters to be used when sending the notification e-mail. Should be present for
    notification handlers. Can be NULL for all non-notification handlers.
    :var notificationPriority: Notification priority to be sent as part of the notification email.
    :var isActive: Status of the subscription to be created.
    Values can be true for ACTIVE and false for INACTIVE.
    
    Events are only generated and notified for the ACTIVE and not-expired subscriptions.
    :var name: Name for the subscription to be created.
    :var eventType: Type of event that will trigger the subscription.
    :var handlers: An ordered list of handlers to execute when processing the Subscription. Handlers will be executed
    in the order they appear in this list. Duplicates are not allowed. Each handler specified will also have its own
    set of handler parameters that will be passed to the handler during its execution.
    :var condition: The Condition to evaluate when processing this Subscription. Only valid for class based
    Subscriptions. Can be NULL.
    :var criteria: Attribute criteria to process when the Subscription is being executed. If these attribute values
    match those of the object, the Subscription will apply. Each entry in the list will be compared to the next using
    the logicOperator of the next value. The logicOperator of the first entry will not be used. Can be empty.
    :var revisionOption: Indicates the type of Item-Revision objects that will apply to the Subscription.
    :var executionPeriod: A Structure to hold executionTime, executionDay and frequency of the subscription.
    :var expirationDate: The date when the Subscription expires and will no longer be processed. Setting NULL will
    cause the Subscription to never expire.
    """
    target: BusinessObject = None
    subscriber: User = None
    notification: NotificationDefinition = None
    notificationPriority: str = ''
    isActive: bool = False
    name: str = ''
    eventType: ImanEventType = None
    handlers: List[ActionHandler] = ()
    condition: Condition = None
    criteria: List[AttributeCriteria] = ()
    revisionOption: RevisionOptions2 = None
    executionPeriod: ExecutionPeriod = None
    expirationDate: datetime = None


@dataclass
class TransferNotificationInput(TcBaseObj):
    """
    TransferNotificationInput holds the data needed to have subscriptions transferred to other users temporarily or
    permanently. In case of temporary transfer a range of dates is provided during which the transfer will be take
    place.
    
    :var subscriptions: List of ImanSubscription objects for bulk transfer.
    :var notifier: The User to be notified.
    :var notificationDateRange: The from and to dates in case of temporary transfer of the subscription.
    """
    subscriptions: List[ImanSubscription] = ()
    notifier: User = None
    notificationDateRange: DateRange = None


@dataclass
class DateRange(TcBaseObj):
    """
    Contains the from and to dates required to find a Subscription.
    
    :var fromDate: The date from which Subscriptions are to be found.
    :var toDate: The date till which Subscriptions are to be found.
    """
    fromDate: datetime = None
    toDate: datetime = None


@dataclass
class EventObject(TcBaseObj):
    """
    The EventObject structure represents required parameter to get event type names for the businessObject. 
    
    :var clientId: A unique identifier supplied by the caller. This ID is client's way of identifying event list. This
    is a required parameter. If nothing is to be passed to clientId; assign an empty String object. Assigning NULL to
    clientId is not allowed.
    :var businessObject: The Business Object  for which the valid Auditable and Subscribable event type list is to be
    retrieved. This is a required parameter.
    """
    clientId: str = ''
    businessObject: BusinessObject = None


class RevisionOptions2(Enum):
    """
    Specifies which level of Item Revisions should be considered for Item based Subscriptions. This only applies to
    Assign Status, Attach and New Item Rev events on target objects of type Item. For example, if a New Item Rev
    Subscription is defined with the BaselineRevisions2 option, only new Baseline Item Revisions will trigger the
    Subscription.
    
    NoRevisions is the default value for events other that those listed above.
    """
    NoRevisions2 = 'NoRevisions2'
    AllRevisions2 = 'AllRevisions2'
    ItemRevisions2 = 'ItemRevisions2'
    BaselineRevision2 = 'BaselineRevision2'



from __future__ import annotations

from tcsoa.gen.Internal.DocumentManagement._2013_05.DigitalSignature import CheckOutForSignResponse
from tcsoa.gen.BusinessObjects import Dataset
from tcsoa.gen.DocumentManagement._2010_04.DigitalSignature import DigtalSigningSaveResponse, DigitalSignSaveInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class DigitalSignatureService(TcService):

    @classmethod
    def isCheckOutForSign(cls, inputDatasets: List[Dataset]) -> CheckOutForSignResponse:
        """
        This operation determines each Dataset (from the provided input list of Dataset objects) is checked out for
        digitally sign. It returns the list of true or false corresponding to order of the input Dataset object list,
        and the ServiceData with any partial errors from the system.
        
        Use cases:
        Use Case 1: Determine a Dataset is checked out for sign
        A user selects a dataset from a client application (for example, Rich Client or Thin Client or Microsoft Office
        client). The Sign action in the client will result in the PDF file being checked out and opened in the signing
        application (for example, Adobe Acrobat or Reader). Internally, the client application initiates this operation
        to determine if the dataset is checked out for digitally sign, if it is true then the client application will
        disable the CheckIn, CheckOut, Transfer Checkout menu.
        """
        return cls.execute_soa_method(
            method_name='isCheckOutForSign',
            library='Internal-DocumentManagement',
            service_date='2013_05',
            service_name='DigitalSignature',
            params={'inputDatasets': inputDatasets},
            response_cls=CheckOutForSignResponse,
        )

    @classmethod
    def digitalSigningSaveTool(cls, saveInput: DigitalSignSaveInput) -> DigtalSigningSaveResponse:
        """
        This operation updates and checks in the input Dataset object (which has the digitally signature named
        reference file). The provided input structure DigitalSignSaveInput (contains Dataset object, File Management
        File (FMS) file ticket, the signing tool name, validation of the signature, the signed user name, and the sign
        time). The signed file must be uploaded to Teamcenter volume prior invoking this operation. The Dataset is
        returned if the update is successful; otherwise, partial error information is returned in the ServiceData.
        
        Use cases:
        Use Case 1: Sign a PDF Dataset
        A user selects a PDF dataset and Sign the dataset from a client application (for example, Rich Client or Thin
        Client or Microsoft Office client). The Sign action in the client will result in the PDF file being checked out
        and opened in the signing application (for example, Adobe Acrobat or Reader). After the user applied the
        digital signature and save the PDF file, this will trigger the signing application to call back to the client
        application, and the client application then initiates this operation to update the PDF datasets named
        reference file (file must be upload to Teamcenter volume first) with the signed PDF file and check in the PDF
        dataset.
        """
        return cls.execute_soa_method(
            method_name='digitalSigningSaveTool',
            library='Internal-DocumentManagement',
            service_date='2013_05',
            service_name='DigitalSignature',
            params={'saveInput': saveInput},
            response_cls=DigtalSigningSaveResponse,
        )

    @classmethod
    def cancelSign(cls, inputDatasets: List[Dataset]) -> ServiceData:
        """
        This operation cancels the previously checked out Dataset objects for digitally sign operation. It returns the
        ServiceData with any partial errors from the system.
        
        Use cases:
        Use Case 1: Cancel Sign a PDF Dataset
        A user selects a PDF dataset and Sign the dataset from a client application (for example, Rich Client or Thin
        Client or Microsoft Office client). The Sign action in the client will result in the PDF file being checked out
        and opened in the signing application (for example, Adobe Acrobat or Reader). The user closes the signing
        application without applying the digital signature. From the client application, the user initiates the Cancel
        Sign action. This will initiates this operation to cancel the checked out of the PDF dataset.
        """
        return cls.execute_soa_method(
            method_name='cancelSign',
            library='Internal-DocumentManagement',
            service_date='2013_05',
            service_name='DigitalSignature',
            params={'inputDatasets': inputDatasets},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import EPMTaskTemplate
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetTaskResultsResponse(TcBaseObj):
    """
    Response from GetTaskResults SOA
    
    :var output: Output of GetTaskResults method
    :var serviceData: Service data
    """
    output: List[TaskResultsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class TaskResultsOutput(TcBaseObj):
    """
    Output of GetTaskResults method
    
    :var taskTemplate: Task template object
    :var taskResults: Task results
    """
    taskTemplate: EPMTaskTemplate = None
    taskResults: List[str] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ImanQuery
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class QueryInput(TcBaseObj):
    """
    The information about each Saved Query to be processed is provided 
    by way of the SavedQueryInput data structure.
    
    :var query: The saved query object to be executed.
    :var entries: Names of search criteria relevant to the query object.
    :var values: Values for the criteria.
    :var limitList: A list of objects (optional) against which the search is conducted,
    if the list is empty, search will be conducted against the database.
    :var maxNumToReturn: A specified maximum number of matches to be returned,
    0 means no limit.
    :var resultsType: The type of results expected from this operation:
    0 (top-level objects only),
    1 (top-level objects plus children: Hierarchical/Indented results),
    2 (default value as specified on the query object).
    """
    query: ImanQuery = None
    entries: List[str] = ()
    values: List[str] = ()
    limitList: List[BusinessObject] = ()
    maxNumToReturn: int = 0
    resultsType: int = 0


@dataclass
class QueryResults(TcBaseObj):
    """
    Results of saved query  execution, including hierarchy information in a relation map for Hierarchical/Indented
    execution.  Hierarchical/Indented execution example1: If the query is: Find all Items where the Item Revisions have
    a Specification Dataset with name = "xyz1", then results will be comprised of subsets, each containing: an Item, a
    qualifying Item Revision and a qualifying Dataset. For each result subset, the corresponding entries in the
    relation map would be 0 (for root Item), 1 (for first-level Item Revision), 2 (for second-level Dataset).  
    Hierarchical/Indented execution example2: If the query is: Find all Item Revisions that have a Specification
    Dataset with name = "xyz1" AND Form of type Item Revision Master with user_data_1 = "xyz2", then the results will
    comprise of subsets, each containing: an Item Revision, a qualifying Dataset, a qualifying Form and a qualifying
    Item Revision Master. For each result subset, the corresponding entries in the relation map would be 0 (for root
    Item Revision), 1 (for first-level Dataset), 1 (for first-level Form) and 2 (for second-level Item Revision Master).
    
    :var objectsRelMap: Used only for Hierarchical/Indented execution. An integer array containing the hierarchy level
    information which maps 1-to-1 to the returned list of objects which is ordered as subsets. The integer code in the
    relation map indicates if the corresponding object in the results set is a root object (=0), or a first-level child
    (=1), or a second-level child (=2) and so on. In the case of child levels, the level information is with respect to
    the immediately preceding root object entry in the mapping. The order of returns is relevant.
    :var objectUIDS: The object UIDs returned from the search.
    """
    objectsRelMap: List[int] = ()
    objectUIDS: List[str] = ()


@dataclass
class SavedQueriesResponse(TcBaseObj):
    """
    Contains a list of business objects found as well as the service data returned.
    
    :var arrayOfResults: A set of QueryResults data structures.
    :var serviceData: The service data.
    """
    arrayOfResults: List[QueryResults] = ()
    serviceData: ServiceData = None



from tcsoa.gen.Qualification._2014_06.services import QualificationManagementService as imp0
from tcsoa.utils import TcService


class QualificationManagementService(TcService):
    appendQualificationLevel = imp0.appendQualificationLevel
    assignUserQualification = imp0.assignUserQualification
    createQualification = imp0.createQualification
    removeQualificationLevel = imp0.removeQualificationLevel
    removeUserQualification = imp0.removeUserQualification
    updateQualification = imp0.updateQualification



from __future__ import annotations

from tcsoa.gen.BusinessObjects import User, BusinessObject, ImanType, IdContext, IdDispRule
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AlternateIdentifiersInput(TcBaseObj):
    """
    Description of Alternate Identifiers to create.
    
    :var context: The IdContext to associate with the Alternate ID.
    :var identifierType: The Type of Alternate Identifier to create.
    :var mainObject: The Item to be associated with the Alternate ID.
    :var revObject: An ItemRevision to associate with the Alternate ID, optional.
    """
    context: IdContext = None
    identifierType: ImanType = None
    mainObject: IdentifierData = None
    revObject: IdentifierData = None


@dataclass
class GetContextAndIdentifiersResponse(TcBaseObj):
    """
    'GetContextAndIdentifierTypes' response
    
    :var contextAndIdentifierTypesMap: A map of context and identifier types for each requested ImanType
    (ImanType/'ContextAndIdentifierTypes')
    :var serviceData: Service data associated with the call
    """
    contextAndIdentifierTypesMap: ContextAndIdentifierTypesMap = None
    serviceData: ServiceData = None


@dataclass
class IdentifierData(TcBaseObj):
    """
    This structure contains information for Identifier data.
    
    :var identifiableObject: identifiableObject
    :var alternateId: alternateId
    :var additionalProps: additionalProps
    :var makeDefault: makeDefault
    """
    identifiableObject: BusinessObject = None
    alternateId: str = ''
    additionalProps: NameValueMap2 = None
    makeDefault: bool = False


@dataclass
class ListAlternateIdDisplayRulesInfo(TcBaseObj):
    """
    Input structure for 'ListAlternateIdDisplayRules' operation
    
    
    :var users: A list of users to return display rules for.
    :var currentUser: Flag to indicate display rules for current user.
    """
    users: List[User] = ()
    currentUser: bool = False


@dataclass
class ListAlternateIdDisplayRulesResponse(TcBaseObj):
    """
    Return structure 'ListAlternateIdDisplayRules'
    
    :var userDisplayRuleMaps: A list of maps of 'Teamcenter::UserImpl' to 'IdDisplayRules'.
    
    :var currentRuleInDB: The current rule in the database ( valid for current user only ).
    :var serviceData: 'ServiceData' associated with the call.
    """
    userDisplayRuleMaps: UserDisplayRuleMap = None
    currentRuleInDB: IdDispRule = None
    serviceData: ServiceData = None


@dataclass
class ValidateAlternateIdInput(TcBaseObj):
    """
    Input structure representing the alternate id to validate.
    
    :var clientId: Input 'clientId' to help the caller match the input to the output and identify error conditions.
    :var alternateId: The Item alternateId to be validated.
    :var alternateRevId: The ItemRevision alternate id to be validated.
    :var patternName: The name of the pattern for the alternate id. This is passed to the user exit
    'USER_validate_alt_id' for validation.
    :var context: The IdContext for the alternate id.
    :var identifierType: The ImanType for the Identifier.
    """
    clientId: str = ''
    alternateId: str = ''
    alternateRevId: str = ''
    patternName: str = ''
    context: IdContext = None
    identifierType: ImanType = None


@dataclass
class ValidateAlternateIdOutput(TcBaseObj):
    """
    This structure contains information for 'ValidateAlternateIdOutput'.
    
    :var clientId: Input 'clientId' to help the caller match the input to the output.
    :var modifiedAltId: Returned alternate id for the Item. This will be a modified id if the input id was invalid.
    :var modifiedAltRevId: Returned alternate id for the ItemRevision. This will be a modified id if the input id was
    invalid.
    :var validityId: Status of the validation for the alternate id for the Item. The values can be found in
    'Teamcenter::Soa::Core::_2007_12::DataManagement::ValidateIdsStatus'
    :var validityRevId: Status of the validation for the alternate id for the ItemRevision. The values can be found in
    'Teamcenter::Soa::Core::_2007_12::DataManagement::ValidateIdsStatus'
    """
    clientId: str = ''
    modifiedAltId: str = ''
    modifiedAltRevId: str = ''
    validityId: ValidateIdsStatus = None
    validityRevId: ValidateIdsStatus = None


@dataclass
class ValidateAlternateIdResponse(TcBaseObj):
    """
    Return structure containing a list of output corresponding to the input and the 'ServiceData'.
    
    :var output: List of 'ValidateAlternateIdOutput'.
    :var serviceData: The 'ServiceData' will contain the objects that are returned by the service call. Any partial
    errors will be mapped with input client id and error message in the 'ServiceData'.
    """
    output: List[ValidateAlternateIdOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ContextAndIdentifierTypes(TcBaseObj):
    """
    This structure contains Context and Identifier Types.
    
    :var context: The IdContext
    :var identfierTypes: The list of ImanTypes that are valid for the IdContext
    """
    context: IdContext = None
    identfierTypes: List[ImanType] = ()


class ValidateIdsStatus(Enum):
    """
    A map of enum of the status for validating ids.
    """
    Valid = 'Valid'
    Invalid = 'Invalid'
    Modified = 'Modified'
    Override = 'Override'
    Duplicate = 'Duplicate'


"""
Map of name and values
"""
NameValueMap2 = Dict[str, List[str]]


"""
A map of user to 'IdDispRule'.
"""
UserDisplayRuleMap = Dict[User, List[IdDispRule]]


"""
Map of IManType to the list of 'ContextAndIdentifierTypes'.
"""
ContextAndIdentifierTypesMap = Dict[ImanType, List[ContextAndIdentifierTypes]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import TaskDependency, TCCalendar, Schedule, BusinessObject, ResourceAssignment, ScheduleTask, TCCalendarEvent
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GenericAttributesContainer(TcBaseObj):
    """
    A collection of KeyValContainers or StringValContainers reprenting the create or updates for a single object,
    such as Schedule, ScheduleTask, TaskDependency, or ResourceAssignment.
    
    :var object: A tag to the object being updated.
    :var keyValueContainer: A collection of updates.
    :var stringValueContainer: A collection of updates.
    """
    object: BusinessObject = None
    keyValueContainer: List[KeyValContainer] = ()
    stringValueContainer: List[StringValContainer] = ()


@dataclass
class KeyValContainer(TcBaseObj):
    """
    The key value container.
    
    :var key: An integer key identifying the attribute  or parameterised index
    :var value: A string representation of the value of the attribute.
    :var type: An integer to help determine the type of object represented by the value.
    """
    key: int = 0
    value: str = ''
    type: int = 0


@dataclass
class LoadProgramViewContainer(TcBaseObj):
    """
    An input container containing the list of schedules and filter conditions.
    
    :var schedules: The UIDs of the Schedules which are included in this program view.
    :var loadProgramViewSets: The collection of filters to be applied.
    """
    schedules: List[str] = ()
    loadProgramViewSets: List[LoadProgramViewSet] = ()


@dataclass
class LoadProgramViewFilter(TcBaseObj):
    """
    The represents the a single input filter in the query for loading the program view.
    
    :var field: The integer value of property to filter out tasks or schedules.
    :var value: The value of the property to filter out schedules or tasks.
    :var condition: The condition: greater-than, less-than etc.
    """
    field: int = 0
    value: str = ''
    condition: int = 0


@dataclass
class LoadProgramViewResponse(TcBaseObj):
    """
    Contains all the necessary response for the program view.  The Schedules, ScheduleTasks, ResourceAssignments, and
    TaskDependencies.
    
    :var schedules: The Schedules.
    :var scheduleTask: The ScheduleTasks.
    :var resourceAssignment: The ResourceAssignments of the scheduleTasks.
    :var taskDependency: The TaskDependecies of the scheduleTasks.
    :var calendar: The Calenders of the schedules.
    :var calendarEvent: The calendar events for the calendars.
    :var serviceData: The ServiceData.
    """
    schedules: List[Schedule] = ()
    scheduleTask: List[ScheduleTask] = ()
    resourceAssignment: List[ResourceAssignment] = ()
    taskDependency: List[TaskDependency] = ()
    calendar: List[TCCalendar] = ()
    calendarEvent: List[TCCalendarEvent] = ()
    serviceData: ServiceData = None


@dataclass
class LoadProgramViewSet(TcBaseObj):
    """
    The collection of input filters
    
    :var loadProgramViewFilters: The filters.
    """
    loadProgramViewFilters: List[LoadProgramViewFilter] = ()


@dataclass
class LoadResourceAssignmentContainer(TcBaseObj):
    """
    A container representing the Query input for the resource load.
    
    :var resource: The resource Tag(s). could be a user or discipline or group. (Currently only user is supported)
    :var schedulesToAlwaysInclude: An optional list of Schedules to always include to search criteria.
    :var startDate: An optional Start Date to filter with (requires endDate).
    :var endDate: An optional Finish Date to filter with (requires startDate).
    """
    resource: List[BusinessObject] = ()
    schedulesToAlwaysInclude: List[Schedule] = ()
    startDate: datetime = None
    endDate: datetime = None


@dataclass
class ResourceAssignmentLoadResponse(TcBaseObj):
    """
    The response container with several SingleAssignmentLoadRespones.
    
    :var serviceData: The ServiceData.
    :var resourceData: The collection of SingleAssignmentLoadResponses.
    """
    serviceData: ServiceData = None
    resourceData: List[SingleAssignmentLoadResponse] = ()


@dataclass
class ScheduleModifyContainer(TcBaseObj):
    """
    The information necessary to update a schedule.  It contains information about new, updated, and deleted tasks,
    resource assignments, and task dependcies
    
    :var scheduleUpdates: The updates to this schedule.
    :var deleteAllTasks: Should it delete all the tasks before beginning?  If this is set, it over-rides everything.
    :var deletedAssignments: A list of resource assignments to delete.
    :var newTasks: The information needed to create new tasks.
    :var newDependencies: The information needed to create new dependencies.
    :var newAssignments: The information needed to create new resource assignments.
    :var taskUpdates: The information needed to update existing tasks.
    :var dependencyUpdates: The information needed to update existing dependencies.
    :var assignmentUpdates: The information needed to update existing resource assignments.
    :var deletedTasks: A list of tasks to delete.
    :var deletedDependencies: A list of dependencies to delete.
    """
    scheduleUpdates: GenericAttributesContainer = None
    deleteAllTasks: bool = False
    deletedAssignments: List[ResourceAssignment] = ()
    newTasks: List[GenericAttributesContainer] = ()
    newDependencies: List[GenericAttributesContainer] = ()
    newAssignments: List[GenericAttributesContainer] = ()
    taskUpdates: List[GenericAttributesContainer] = ()
    dependencyUpdates: List[GenericAttributesContainer] = ()
    assignmentUpdates: List[GenericAttributesContainer] = ()
    deletedTasks: List[ScheduleTask] = ()
    deletedDependencies: List[TaskDependency] = ()


@dataclass
class SingleAssignmentLoadResponse(TcBaseObj):
    """
    The container representing a single resources's assignment output data which matched the query.
    
    :var resource: The resource (could be a user or discipline or group).
    :var resourceAssignment: The resource assignments referencing the resource.
    :var schedules: The schedules which contain the resourceAssignments and tasks.
    :var tasks: The assigned tasks.
    :var calendars: The calendars referenced in the assignment (can be base, resource, schedule and schedule member
    calendars).
    :var calendarEvents: The events referenced in the calendars.
    """
    resource: BusinessObject = None
    resourceAssignment: List[ResourceAssignment] = ()
    schedules: List[Schedule] = ()
    tasks: List[ScheduleTask] = ()
    calendars: List[TCCalendar] = ()
    calendarEvents: List[TCCalendarEvent] = ()


@dataclass
class StringValContainer(TcBaseObj):
    """
    The container represents a single attribute's value. 
    The name of the attribute is represented by the key field. 
    The data type of the attribute is represented by the type field. The type field can be one of the following;
    char = 0;
    date = 1;
    double = 2;
    float = 3;
    int = 4;
    bool = 5;
    short = 6;
    string = 7;
    ModelObject = 8;
    
    :var key: A string key identifying the attribute.
    :var value: A string representation of the value of the attribute.
    :var type: An integer to help determine the type of object represented by the value.
    """
    key: str = ''
    value: str = ''
    type: int = 0



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, TCCalendar, TCCalendarEvent
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetTCCalendarData(TcBaseObj):
    """
    The query information for calendars
    
    :var type: The type of TCCalendar (1-System Calendar, 2-Schedule Calendar, 3-User Calendar, 4-ScheduleMember
    Calendar)
    :var schedule: The related schedule
    :var resource: The related resource
    """
    type: int = 0
    schedule: BusinessObject = None
    resource: BusinessObject = None


@dataclass
class NewTCCalendar(TcBaseObj):
    """
    Holder for new tccalendar and its events
    
    :var tccalendarAttributes: The calendar data
    :var newEvents: The calendar events.
    """
    tccalendarAttributes: TCCalendarData = None
    newEvents: List[TCCalendarEventData] = ()


@dataclass
class TCCalendarContainer(TcBaseObj):
    """
    Holder for TCCalendar and its events
    
    :var tccalendar: The TCCalendar
    :var events: The TCCalendarEvents for the TCCalendar.
    """
    tccalendar: TCCalendar = None
    events: List[TCCalendarEvent] = ()


@dataclass
class TCCalendarData(TcBaseObj):
    """
    Data for a calendar
    
    :var name: Name of the tccalendar
    :var description: Description of the tccalendar
    :var minsFri: Default minutes for Friday
    :var minsSat: Default minutes for Saturday
    :var schedule: Null indicates no schedule
    :var source: TCCalendar source(schedule/resource)
    :var type: Type of tccalendar
    :var minsSun: Default minutes for Sunday
    :var minsMon: Default minutes for Monday
    :var minsTue: Default minutes for Tuesday
    :var minsWed: Default minutes for Wednesday
    :var minsThu: Default minutes for Thursday
    """
    name: str = ''
    description: str = ''
    minsFri: int = 0
    minsSat: int = 0
    schedule: BusinessObject = None
    source: BusinessObject = None
    type: int = 0
    minsSun: int = 0
    minsMon: int = 0
    minsTue: int = 0
    minsWed: int = 0
    minsThu: int = 0


@dataclass
class TCCalendarEventData(TcBaseObj):
    """
    Data for a calendar event.
    
    :var eventDate: The date on which event occurs
    :var eventMin: The duration of the event in minutes
    """
    eventDate: datetime = None
    eventMin: int = 0


@dataclass
class TCCalendarModification(TcBaseObj):
    """
    Modification structure
    
    :var scheduleTCCalendar: TCCalendar to modify
    :var action: enumerated action
    """
    scheduleTCCalendar: TCCalendar = None
    action: ActionString = None


@dataclass
class TCCalendarResponse(TcBaseObj):
    """
    Standard response for most tccalendar calls
    
    :var tccalendars: The calendar containers
    :var serviceData: The service data
    """
    tccalendars: List[TCCalendarContainer] = ()
    serviceData: ServiceData = None


@dataclass
class TCCalendarUpdate(TcBaseObj):
    """
    A calendar update
    
    :var tccalendar: The calendar being updated
    :var updatedAttributes: The updated attributes
    :var eventsToDelete: Events to delete
    :var eventsToUpdate: Events to update
    :var eventAttributes: Corresponding updates for the EventsToUpdate
    :var newEvents: Events to create
    """
    tccalendar: TCCalendar = None
    updatedAttributes: TCCalendarData = None
    eventsToDelete: List[TCCalendarEvent] = ()
    eventsToUpdate: List[TCCalendarEvent] = ()
    eventAttributes: List[TCCalendarEventData] = ()
    newEvents: List[TCCalendarEventData] = ()


class ActionString(Enum):
    """
    ActionString
    """
    MergeSchedule = 'MergeSchedule'
    ResetSchedule = 'ResetSchedule'



from __future__ import annotations

from tcsoa.gen.Internal.VendorManagementAW._2019_12.VendorManagement import AssignPartnerContractInput, RemovePartnerContractInput, ColumnInfoResponse, ColumnInfoInput, ViewModelRowsResponse
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService
from tcsoa.gen.BusinessObjects import WorkspaceObject


class VendorManagementService(TcService):

    @classmethod
    def assignPartnerContractToObject(cls, input: AssignPartnerContractInput) -> ServiceData:
        """
        This operation assigns the partner contract to the given WorkspaceObject object as specified in the input. The
        selected partner contract will be assigned to the given context information.
        """
        return cls.execute_soa_method(
            method_name='assignPartnerContractToObject',
            library='Internal-VendorManagementAW',
            service_date='2019_12',
            service_name='VendorManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def getColumnInfo(cls, input: ColumnInfoInput) -> ColumnInfoResponse:
        """
        This operation returns the information of the columns for a given type of table to render a business object.
        
        The column information returned as part of this operation includes display name, internal name, object type, 
        width and a flag to indicate if the column needs to be displayed or not.
        """
        return cls.execute_soa_method(
            method_name='getColumnInfo',
            library='Internal-VendorManagementAW',
            service_date='2019_12',
            service_name='VendorManagement',
            params={'input': input},
            response_cls=ColumnInfoResponse,
        )

    @classmethod
    def getPartnerContractsOfSelectedObject(cls, inputObject: WorkspaceObject) -> ViewModelRowsResponse:
        """
        This operation returns the information of the partner contracts of the selected object.
        """
        return cls.execute_soa_method(
            method_name='getPartnerContractsOfSelectedObject',
            library='Internal-VendorManagementAW',
            service_date='2019_12',
            service_name='VendorManagement',
            params={'inputObject': inputObject},
            response_cls=ViewModelRowsResponse,
        )

    @classmethod
    def removePartnerContractFromObject(cls, removePartnerContractInput: RemovePartnerContractInput) -> ServiceData:
        """
        This operation removes the partner contract from the given WorkspaceObject object as specified in the input.
        The selected partner contract will be removed from the given context information.
        """
        return cls.execute_soa_method(
            method_name='removePartnerContractFromObject',
            library='Internal-VendorManagementAW',
            service_date='2019_12',
            service_name='VendorManagement',
            params={'removePartnerContractInput': removePartnerContractInput},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, BillRate, ScheduleTask, Schedule
from tcsoa.gen.ProjectManagement._2008_06.ScheduleManagement import FixedCostContainer
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class FailedObjectContainer(TcBaseObj):
    """
    FailedObjectContainer will hold the Schedule Task and an error message of the operation that failed.
    
    :var tasks: A vector of Schedule Tasks
    :var errorMsgs: A vector of strings that have the error messages.
    :var serviceData: The service data
    """
    tasks: List[ScheduleTask] = ()
    errorMsgs: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class AnalyticsMultipleResourceAssignmentStacks(TcBaseObj):
    """
    Reponse container for resource graph data request that contains one or more resources.
    
    :var stacksVector: Multiple resource stacks container.
    """
    stacksVector: List[AnalyticsResourceAssignmentStacks] = ()


@dataclass
class AnalyticsResourceAssignmentDayStack(TcBaseObj):
    """
    Holds data for all the assigned tasks for the day. 
    
    :var numElements: Number of elements that makeup the stack for the day.
    :var stack: Stack/pile of assigned tasks for the day.
    """
    numElements: int = 0
    stack: List[AnalyticsResourceAssignmentSingleTask] = ()


@dataclass
class AnalyticsResourceAssignmentSingleTask(TcBaseObj):
    """
    Information on single task assignment for the day.
    
    :var day: Date of assignments.
    :var userUid: Uid of the business object
    :var taskEndDate: ScheduleTask end date.
    :var taskMinutes: Amount of work on the task.
    :var resourceMinutes: Day's work performed on the task.
    :var stackBase: Where to place the next element in the day's stack of work.
    :var taskMinutesOverLoad: Amount of day's work over and above what the user's calendar indicates.
    :var userData: Descriiption of the data to provide tooltip information.
    :var calUid: Uid of user calendar
    :var schUid: Uid of Schedule to which task assignment belongs.  
    :var schTaskUid: The scheduleTask from which the assignment is assigned. 
    :var schName: Name of the schedule
    :var schStartDate: Schedule start date
    :var schEndDate: Schedule end/finish date. 
    :var taskName: Name of the scheduleTask from which the assignment was made.
    :var taskstartDate: ScheduleTask start date.
    """
    day: datetime = None
    userUid: str = ''
    taskEndDate: datetime = None
    taskMinutes: float = 0.0
    resourceMinutes: float = 0.0
    stackBase: float = 0.0
    taskMinutesOverLoad: float = 0.0
    userData: str = ''
    calUid: str = ''
    schUid: str = ''
    schTaskUid: str = ''
    schName: str = ''
    schStartDate: datetime = None
    schEndDate: datetime = None
    taskName: str = ''
    taskstartDate: datetime = None


@dataclass
class AnalyticsResourceAssignmentStacks(TcBaseObj):
    """
    Holds stack of tasks for each day and for all days between the start and finish dates.
    
    :var userName: Name of the reesource
    :var numDays: Numebr of days between the start and finish range.The range represent when the graph starts and ends.
    :var stacks: Holds data for all the days that falls between start and finish dates. 
    """
    userName: str = ''
    numDays: int = 0
    stacks: List[AnalyticsResourceAssignmentDayStack] = ()


@dataclass
class InsertScheduleContainer(TcBaseObj):
    """
    Contains the information of the schedule being inserted, the master schedule, the task in the master schedule below
    which the sub-schedule is inserted and the Boolean to adjust the master dates
    
    :var subSchedule: The schedule that is being inserted in the master schedule.
    :var masterSchedule: The schedule in which the subSchedule is being inserted.
    :var masterScheduleTask: The task in the master schedule below which the subSchedule is being inserted.
    :var adjustMasterDates: Boolean value of true will allow the master start and/or end date to automatically adjust
    if the sub schedule start and/or finish date do not lie between the master dates.
    """
    subSchedule: Schedule = None
    masterSchedule: Schedule = None
    masterScheduleTask: ScheduleTask = None
    adjustMasterDates: bool = False


@dataclass
class LoadResourceGraphContainer(TcBaseObj):
    """
    A container representing the Query input for the resource load.
    
    :var resources: The user Tags.
    :var startDate: An optional Start Date to filter with requires endDate.
    :var endDate: An optional Finish Date to filter with requires startDate.
    """
    resources: List[BusinessObject] = ()
    startDate: datetime = None
    endDate: datetime = None


@dataclass
class CostDetailContainer(TcBaseObj):
    """
    The container that has the details of the cost attributes of schedule or task 
    
    :var totalEstimatedMinutes: The total estimated minutes of schedule or collection of selected schedule tasks or
    individual task
    :var totalAccruedMinutes: The total accrued minutes of schedule or collection of selected schedule tasks or
    individual task .
    :var numchildren: the number of children associated with the summary tasks or schedule
    :var fixedCostData: Fixed cost container
    :var taskCostDetails: the cost details of each sub-task
    :var totalEstimatedCost: The total estimated cost of schedule task
    :var totalAccruedCost: The total estimated cost of schedule task
    :var billCode: The bill code for task. Valid values are determined by the LOV. Valid values are {unassigned,
    General, Management, ProjectMgmt, Sales, Training, Travel, ProductDev, SoftwareDev}. 
    :var billSubCode: The bill subcode for the task. Valid values are determined by the LOV. Valid value are following
    groups depending upon billcode.eg. If billcode is General then you can mentioned subcode as either of following set
    (unassigned, Billable, Billed, Standard, Unbillable}
    :var billingType: The billing type of the task. Valid values are determined by the LOV. Valid value are {
    unassigned, Billable, Billed, Standard, Unbillable}.
    :var billRate: The name of the bill rate associated with the task. Valid value can be empty string
     
    
    :var currency: The currency used in the cost for schedule or task.
    :var task: The task which contains the total costs (sum of fixed costs and variable costs) 
    """
    totalEstimatedMinutes: int = 0
    totalAccruedMinutes: int = 0
    numchildren: int = 0
    fixedCostData: List[FixedCostContainer] = ()
    taskCostDetails: List[CostDetailContainer] = ()
    totalEstimatedCost: float = 0.0
    totalAccruedCost: float = 0.0
    billCode: str = ''
    billSubCode: str = ''
    billingType: str = ''
    billRate: BillRate = None
    currency: str = ''
    task: ScheduleTask = None


@dataclass
class CostDetailResponse(TcBaseObj):
    """
    The response of the get cost roll up data
    
    :var costDetails: The response of the get cost roll up data
    :var costServiceData: Service data
    """
    costDetails: List[CostDetailContainer] = ()
    costServiceData: ServiceData = None


@dataclass
class DetachScheduleContainer(TcBaseObj):
    """
    DetachScheduleContainer    Contains the information of the schedule being detached, the master schedule.
    
    :var subSchedule: The schedule that is being detached from the master schedule.
    :var masterSchedule: The master schedule from which the subSchedule is being detached.
    """
    subSchedule: Schedule = None
    masterSchedule: Schedule = None


@dataclass
class EVMDataRequest(TcBaseObj):
    """
    Earned Value Data Request of schedule or task 
    
    :var taskVector: The tag of a task
    :var calcBasisSelected: To view the earned value data of schedule or task by cost , use the boolean value as false
    . To view the earned value data of schedule or task by duration , use the boolean value as true and the default
    option .
    :var calcWorkComplete: To view the earned value data based on work complete of schedule's or tasks's percentage
    complete used, use the boolean value as true and the default option .To view the earned value data based on the
    work complete of schedule's task's actual hours used , use the boolean value as false . 
    :var selectedLabel: To view the earned value data based on the schedule's or task's earned value management labels
    , use the boolean value as true and the default option . To view the earned value data based on the schedule's or
    task's Cost/Schedule control systems criteria labels , use the boolean value as false.
    """
    taskVector: List[ScheduleTask] = ()
    calcBasisSelected: bool = False
    calcWorkComplete: bool = False
    selectedLabel: bool = False


@dataclass
class EVMResultsContainer(TcBaseObj):
    """
    The container that contains the task&apos;s or schedule&apos;s earned value data.
    
    :var task: The tag of a task 
    :var evmResults: The evm results that holds the earned value calculations of schedule or task
    """
    task: ScheduleTask = None
    evmResults: EVMResults = None


@dataclass
class EVMResultsResponse(TcBaseObj):
    """
    The response that contains the evm results 
    
    :var evmData: The container that holds the earned value data of the schedule or task
    :var evmServiceData: The service data 
    """
    evmData: EVMResultsContainer = None
    evmServiceData: ServiceData = None


"""
The earned value map holds the data of each earned value paramater used in earmed valued calculation of schedule or task .Planned value(PV) is the total budgeted costs for the project that is scheduled to be doneuntil now
"""
EVMResults = Dict[str, float]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ItemRevision, Folder, BOMWindow
from tcsoa.gen.Cad._2007_01.StructureManagement import ExpandPSAllLevelsResponse, ExpandPSOneLevelPref, DeleteAssemblyArrangementsInfo, DeleteAssemblyArrangementsResponse, CreateOrUpdateAbsoluteStructureInfo, CreateOrUpdateAbsoluteStructurePref, GetVariantRulesResponse, ExpandPSAllLevelsPref, CreateOrUpdateAbsoluteStructureResponse, CreateBOMWindowsInfo, CreateOrUpdateRelativeStructurePref, DeleteRelativeStructurePref, CreateOrUpdateRelativeStructureResponse, GetRevisionRulesResponse, ExpandPSAllLevelsInfo, DeleteRelativeStructureInfo, CloseBOMWindowsResponse, CreateBOMWindowsResponse, ExpandPSOneLevelInfo, DeleteRelativeStructureResponse, CreateOrUpdateRelativeStructureInfo, ExpandPSOneLevelResponse
from tcsoa.gen.Cad._2007_01.DataManagement import ExpandGRMRelationsResponse, GetAllAttrMappingsResponse, ResolveAttrMappingsForDatasetResponse, CreateOrUpdateRelationsInfo, ResolveAttrMappingsForDatasetInfo, ExpandPrimaryObjectsResponse, ExpandFolderForCADPref, GetAttrMappingsForDatasetTypeResponse, GenerateAlternateIdsProperties, CreateOrUpdatePartsResponse, CreateOrUpdateRelationsResponse, GetAvailableTypesResponse, CreateOrUpdateRelationsPref, PartInfo, GenerateAlternateIdsResponse, ExpandGRMRelationsPref, ExpandFoldersForCADResponse, ExpandPrimaryObjectsPref, GetAttrMappingsForDatasetTypeCriteria
from typing import List
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def getRevisionRules(cls) -> GetRevisionRulesResponse:
        """
        The GetRevisionRules service gets all the persistent revision rules in the database. It along with the revision
        rules returns its runtime configuration properties status of being fixed or not.
        
        Exceptions:
        >ServiceException Thrown if the system fails retreiving the list of revision rules.
        """
        return cls.execute_soa_method(
            method_name='getRevisionRules',
            library='Cad',
            service_date='2007_01',
            service_name='StructureManagement',
            params={},
            response_cls=GetRevisionRulesResponse,
        )

    @classmethod
    def getVariantRules(cls, itemRevs: List[ItemRevision]) -> GetVariantRulesResponse:
        """
        The GetVariantRules service gets all the variant rules in the database for the given Item Revision. To get
        Product Configurator authored variant rules, value of preference PSM_default_configurator_context must be true.
        
        Exceptions:
        >ServiceException Thrown if the there are no ItemRevision objects input.
        """
        return cls.execute_soa_method(
            method_name='getVariantRules',
            library='Cad',
            service_date='2007_01',
            service_name='StructureManagement',
            params={'itemRevs': itemRevs},
            response_cls=GetVariantRulesResponse,
        )

    @classmethod
    def closeBOMWindows(cls, bomWindows: List[BOMWindow]) -> CloseBOMWindowsResponse:
        """
        Closes a BOMWindow.  Must be used to close the BOMWindow created during Create BOM Window after calls to Expand
        Product Structure services are complete.
        """
        return cls.execute_soa_method(
            method_name='closeBOMWindows',
            library='Cad',
            service_date='2007_01',
            service_name='StructureManagement',
            params={'bomWindows': bomWindows},
            response_cls=CloseBOMWindowsResponse,
        )

    @classmethod
    def createBOMWindows(cls, info: List[CreateBOMWindowsInfo]) -> CreateBOMWindowsResponse:
        """
        Creates a BOMWindow and sets the input Item Revision as the top line.  Can be used to create the BOMLine for
        input to Expand Product Structure services.  All BOMLines under this window are unpacked.  To use the
        Teamcenter default unitNo or use your input RevisionRule with no changes, you must set unitNo to -1 in
        RevisionRuleEntryProps::unitNo.  If it is not specified, your input rule will function as a modified/transient
        revision rule with a unitNo of 0.
        """
        return cls.execute_soa_method(
            method_name='createBOMWindows',
            library='Cad',
            service_date='2007_01',
            service_name='StructureManagement',
            params={'info': info},
            response_cls=CreateBOMWindowsResponse,
        )

    @classmethod
    def createOrUpdateAbsoluteStructure(cls, info: List[CreateOrUpdateAbsoluteStructureInfo], bomViewTypeName: str, complete: bool, pref: CreateOrUpdateAbsoluteStructurePref) -> CreateOrUpdateAbsoluteStructureResponse:
        """
        CreateOrUpdateAbsoluteStucture allows the user to find or create the absolute structure network of objects and
        relations for the input model.
        The service first attempts to check for the presence of duplicate context objects and then validate the
        existence of the structure to be created.
        If any of the objects exist and the input attribute values differ from those already set, an attempt is made to
        update the values.
        Note: The following AbsOccData attributes are not supported for arrangement qualified overrides.
        These attributes can only be overridden at the bvr level (which applies to all arrangements).
        If these attributes are overridden in the AssemblyArrangementInfo, they will be ignored.
        1.child item
        2.GDE object
        3.instance number
        4.occurrence name
        5.note
        6.occurrence type
        7.Variant instance
        As we process one contextItemRev object at one time, it is assumed that all edits for a given contextItemRev
        come in as one set of input.
        
        Exceptions:
        >Service Exception    Thrown if an invalid BOM view type is specified in bomViewTypeName. 
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateAbsoluteStructure',
            library='Cad',
            service_date='2007_01',
            service_name='StructureManagement',
            params={'info': info, 'bomViewTypeName': bomViewTypeName, 'complete': complete, 'pref': pref},
            response_cls=CreateOrUpdateAbsoluteStructureResponse,
        )

    @classmethod
    def createOrUpdateRelativeStructure(cls, inputs: List[CreateOrUpdateRelativeStructureInfo], bomViewTypeName: str, complete: bool, pref: CreateOrUpdateRelativeStructurePref) -> CreateOrUpdateRelativeStructureResponse:
        """
        Create or update the relative structure objects and relations for the input model.
        The service first attempts to check for the presence of duplicate context objects and then validate the
        existence of the structure to be created.
        If any of the objects exist but the input attribute values that differ from those already set, an attempt is
        made to update the values if possible.
        This service assumes the input is in a bottom-up format such that if any failures occur, the structure that is
        created will be consistent.
        No attempt is made in the service to rearrange the input and process it in the correct order.
        As we process one parent context object at one time, it is assumed that all edits for a given parent context
        come in as one set of input.
        
        Exceptions:
        >Service exceptions are thrown for the following error situations:
        
        215005: The BOM view revision (BVR) is not checked out by the current user.
        215006: The BOM view revision is checked out by another user.
        215009: The object type name specified in the input preference does not exist.
        215010: The input CAD occurrence identifier does not correspond to a valid BOM line. This is an error when the
        complete input is false.
        
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateRelativeStructure',
            library='Cad',
            service_date='2007_01',
            service_name='StructureManagement',
            params={'inputs': inputs, 'bomViewTypeName': bomViewTypeName, 'complete': complete, 'pref': pref},
            response_cls=CreateOrUpdateRelativeStructureResponse,
        )

    @classmethod
    def deleteAssemblyArrangements(cls, info: List[DeleteAssemblyArrangementsInfo], bomViewTypeName: str) -> DeleteAssemblyArrangementsResponse:
        """
        Deletes assembly arrangements and all the absolute occurrence data associated with the assembly arrangements,
        it also delete the relation between assembly arrangements and bvr.
        
        Exceptions:
        >Service Exception    Thrown if an invalid BOM view type is specified in bomViewTypeName. 
        """
        return cls.execute_soa_method(
            method_name='deleteAssemblyArrangements',
            library='Cad',
            service_date='2007_01',
            service_name='StructureManagement',
            params={'info': info, 'bomViewTypeName': bomViewTypeName},
            response_cls=DeleteAssemblyArrangementsResponse,
        )

    @classmethod
    def deleteRelativeStructure(cls, inputs: List[DeleteRelativeStructureInfo], bomViewTypeName: str, pref: DeleteRelativeStructurePref) -> DeleteRelativeStructureResponse:
        """
        Deletes one or more first level children of a parent assembly.
        
        Exceptions:
        >Service Exception    Thrown if an invalid BOM view type is specified in bomViewTypeName. 
        """
        return cls.execute_soa_method(
            method_name='deleteRelativeStructure',
            library='Cad',
            service_date='2007_01',
            service_name='StructureManagement',
            params={'inputs': inputs, 'bomViewTypeName': bomViewTypeName, 'pref': pref},
            response_cls=DeleteRelativeStructureResponse,
        )

    @classmethod
    def expandPSAllLevels(cls, input: ExpandPSAllLevelsInfo, pref: ExpandPSAllLevelsPref) -> ExpandPSAllLevelsResponse:
        """
        Finds the chilren at all levels given parent bomlines. Also if required gets the objects of given type and
        relation that are attached to the parent and children
        """
        return cls.execute_soa_method(
            method_name='expandPSAllLevels',
            library='Cad',
            service_date='2007_01',
            service_name='StructureManagement',
            params={'input': input, 'pref': pref},
            response_cls=ExpandPSAllLevelsResponse,
        )

    @classmethod
    def expandPSOneLevel(cls, input: ExpandPSOneLevelInfo, pref: ExpandPSOneLevelPref) -> ExpandPSOneLevelResponse:
        """
        Finds the first level chilren of given parent bomlines. Also if required gets the objects of given type and
        relation that are attached to the parent and children
        """
        return cls.execute_soa_method(
            method_name='expandPSOneLevel',
            library='Cad',
            service_date='2007_01',
            service_name='StructureManagement',
            params={'input': input, 'pref': pref},
            response_cls=ExpandPSOneLevelResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def resolveAttrMappingsForDataset(cls, info: List[ResolveAttrMappingsForDatasetInfo]) -> ResolveAttrMappingsForDatasetResponse:
        """
        Retrieves CAD attribute mapped properties for one or more datasets.
        
        Use cases:
        User does a FileOpen of Teamcenter Item or Dataset (CAD Part).  CAD properties mapped to Teamcenter attribute
        values are shown in the Part attribute display.
        """
        return cls.execute_soa_method(
            method_name='resolveAttrMappingsForDataset',
            library='Cad',
            service_date='2007_01',
            service_name='DataManagement',
            params={'info': info},
            response_cls=ResolveAttrMappingsForDatasetResponse,
        )

    @classmethod
    def createOrUpdateParts(cls, info: List[PartInfo]) -> CreateOrUpdatePartsResponse:
        """
        CreateOrUpdateParts allows the user to create or update a set of parts using Items, Item Revisions, Datasets
        and ExtraObjects.
        If the user supplies a valid Item object reference without specifying a valid item revision object reference or
        id, then only the item will be updated.
        If the user specifies a valid item object reference with a null item revision object reference and a non-null
        revision id, then a new item revision will be created and attached to the item with no relations from the new
        item revision to the previous item revision.
        If the user specifies a valid item object reference and a valid item revision object reference, then the item
        and item revision and related objects will be updated.
        If no item object reference or item revision object references are specified then a new item and item revision
        and related objects will be created.
        All objects created and updated will be returned in the ServiceData.
        All partial errors will contain the clientIDs for all items related to the error message, i.e. if a dataset
        encounters an error, then the ID for that erro will be the item client ID concantentated with the item revision
        id contantenated with the dataset client ID, all separated by semi-colons ( ; ).
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateParts',
            library='Cad',
            service_date='2007_01',
            service_name='DataManagement',
            params={'info': info},
            response_cls=CreateOrUpdatePartsResponse,
        )

    @classmethod
    def createOrUpdateRelations(cls, info: List[CreateOrUpdateRelationsInfo], complete: bool, pref: CreateOrUpdateRelationsPref) -> CreateOrUpdateRelationsResponse:
        """
        'createOrUpdateRelations' creates or updates GRM relations between existing objects in Teamcenter. If the
        'complete' flag is set and a filter is supplied, then any relation types that exist for primary objects in the
        info that are listed in the filter, but the relations are not sent in the input, those relations will be
        deleted.
        
        Use cases:
        The user wishes to relate two objects in Teamcenter. The user specifies the primary and secondary objects, the
        type of relation to be created and any optional data the user wants added to the relation.
        The user wishes to modify the user data on the relationship between two objects. The user specifies the primary
        and secondary objects and the existing relationship. The user also specifies the new user data to be placed on
        the relationship.
        The user wishes to relate two objects in Teamcenter and remove any existing relationships between the objects.
        The user specifies the primary and secondary objects, the type of relation to be created and any optional data
        the user wants added to the relation. The user also sets the 'complete' option to true to delete the existing
        relationships that pass the supplied filter, but does not send those existing relationships in the input.  For
        example, there is an item revision and a dataset related with an IMAN_specification relationship in Teamcenter
        and the user wants to change this to an IMAN_Rendering relationship. The user can specify the item revision and
        the dataset, specify the new relationship is IMAN_Rendering and set the 'complete' flag. With the filter
        specifying the relation type of IMAN_specification and the object type as dataset. This will delete the current
        relationship and create the new one.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateRelations',
            library='Cad',
            service_date='2007_01',
            service_name='DataManagement',
            params={'info': info, 'complete': complete, 'pref': pref},
            response_cls=CreateOrUpdateRelationsResponse,
        )

    @classmethod
    def expandFoldersForCAD(cls, folders: List[Folder], pref: ExpandFolderForCADPref) -> ExpandFoldersForCADResponse:
        """
        The purpose of this service is to provide the contents of a folder that a CAD integration typically needs in
        one service call as opposed to multiple expand calls.
        This service is specifically for Folder expansion and will only return Items, Item Revisions and Folders that
        are contained in the input Folder.
        Other types of objects (Forms, Datasets, etc...) that are contained directly under the input folder will not be
        returned.
        The service will also return the Item Revisions and Datasets for the Items found in the folder and Datasets
        found for the Item Revisions found directly under the folder.
        The Item Revisions returned, are controlled thru an input latestNRevs specifying how many latest revisions
        should be sent to output.
        The Items, Item Revisions and Datasets returned can be filtered thru an input preference of a list of relation
        names and dataset types filter.
        """
        return cls.execute_soa_method(
            method_name='expandFoldersForCAD',
            library='Cad',
            service_date='2007_01',
            service_name='DataManagement',
            params={'folders': folders, 'pref': pref},
            response_cls=ExpandFoldersForCADResponse,
        )

    @classmethod
    def expandGRMRelations(cls, objects: List[BusinessObject], pref: ExpandGRMRelationsPref) -> ExpandGRMRelationsResponse:
        """
        Returns the secondary objects related to the input object for the given list of relation names and other side
        object types filter.
        If no relation names or other side objects types are provided in the input, then all related objects will be
        returned.
        In addition, for performance, if an Item is the output of the expansion, then its associated Item Revisions and
        the Datasets for those Item Revisions will be returned.
        Similarly, if an Item Revision is the output of the expansion, then its associated Datasets will be returned.
        However this additional expansion is controlled through the expItemRev flag.
        """
        return cls.execute_soa_method(
            method_name='expandGRMRelations',
            library='Cad',
            service_date='2007_01',
            service_name='DataManagement',
            params={'objects': objects, 'pref': pref},
            response_cls=ExpandGRMRelationsResponse,
        )

    @classmethod
    def expandPrimaryObjects(cls, objects: List[BusinessObject], pref: ExpandPrimaryObjectsPref) -> ExpandPrimaryObjectsResponse:
        """
        Returns the secondary objects related to the input object for the given list of relation names and other side
        object types filter.
        If no relation names or other side objects types are provided in the input, then all related objects will be
        returned.
        In addition, for performance, if an Item is the output of the expansion, then its associated Item Revisions and
        the Datasets for those Item Revisions will be returned.
        Similarly, if an Item Revision is the output of the expansion, then its associated Datasets will be returned.
        However this additional expansion is controlled through the expItemRev flag.
        """
        return cls.execute_soa_method(
            method_name='expandPrimaryObjects',
            library='Cad',
            service_date='2007_01',
            service_name='DataManagement',
            params={'objects': objects, 'pref': pref},
            response_cls=ExpandPrimaryObjectsResponse,
        )

    @classmethod
    def generateAlternateIds(cls, input: List[GenerateAlternateIdsProperties]) -> GenerateAlternateIdsResponse:
        """
        Generate a list of alternate ids. An alternate id is a displayable id for an Item or ItemRevision that is
        controlled by the user via display rules. The client creates Identifiers that contain the various alternate ids
        to be displayed. Each Identifier is controlled by a display rule. When a display rule is active then the
        appropriate alternate id is displayed.
        
        Use cases:
        The client defines several alternate ids for a part. One alternate id is for the manufacturer of the part( this
        would be their part number ), another would be for a customer ( their part number ) and maybe one for a second
        customer ( again, another part number ). This service allows the client to invoke the user exit
        USER_new_alt_id. This exit will be called once for each count specified in the input using the data passed in
        as parameters.
        """
        return cls.execute_soa_method(
            method_name='generateAlternateIds',
            library='Cad',
            service_date='2007_01',
            service_name='DataManagement',
            params={'input': input},
            response_cls=GenerateAlternateIdsResponse,
        )

    @classmethod
    def getAllAttrMappings(cls) -> GetAllAttrMappingsResponse:
        """
        Retrieves all CAD attribute mapping definitions. Additionally, if a CadAttributeMappingDefinition object has a
        path that includes a GRM or NR part, the associated PropertyDescriptor and any attached ListOfValues objects
        will be returned.
        
        Exceptions:
        >Service Exception    Thrown if any Teamcenter subsystem errors occur during the retrieval of all attribute
        mappings, finding the dataset type or finding the attribute mapping type.
        """
        return cls.execute_soa_method(
            method_name='getAllAttrMappings',
            library='Cad',
            service_date='2007_01',
            service_name='DataManagement',
            params={},
            response_cls=GetAllAttrMappingsResponse,
        )

    @classmethod
    def getAttrMappingsForDatasetType(cls, info: List[GetAttrMappingsForDatasetTypeCriteria]) -> GetAttrMappingsForDatasetTypeResponse:
        """
        This operation retrieves the CAD attribute mapping definitions for one or more dataset type and item type
        combinations.  If a 'CadAttributeMappingDefinition' object has a path that includes a Generic Relationship
        Manager (GRM) or named reference part, the associated property descriptor and any attached 'ListOfValues' (LOV)
        objects will be returned.
        
        Since this operation returns existing Teamcenter attribute mappings, please reference the Teamcenter help
        section on creating attribute mappings.
        
        
        Use cases:
        User needs to have attribute mappings defined in Teamcenter.
        
        For this operation, the dataset object type is passed as input. The client application uses the list of
        returned attribute mapping definitions to present the CAD attributes to the user that correspond to the correct
        Teamcenter attributes.
        
        
        Exceptions:
        >Service Exception    Thrown if any Teamcenter subsystem errors occur during the retrieval of all attribute
        mappings.
        """
        return cls.execute_soa_method(
            method_name='getAttrMappingsForDatasetType',
            library='Cad',
            service_date='2007_01',
            service_name='DataManagement',
            params={'info': info},
            response_cls=GetAttrMappingsForDatasetTypeResponse,
        )

    @classmethod
    def getAvailableTypes(cls, classes: List[str]) -> GetAvailableTypesResponse:
        """
        Finds types implemented by the given input class name.
        
        Use cases:
        User selects File New Item and is presented with a list of creatable item types.
        """
        return cls.execute_soa_method(
            method_name='getAvailableTypes',
            library='Cad',
            service_date='2007_01',
            service_name='DataManagement',
            params={'classes': classes},
            response_cls=GetAvailableTypesResponse,
        )



from __future__ import annotations

from tcsoa.gen.Core._2010_09.DataManagement import CreateOrUpdateStaticTableDataResponse, SetPropertyResponse, PropInfo, StaticTableInfo, EventTypesResponse, PostEventObjectProperties, EventObject, RowData, VerifyExtensionInfo, VerifyExtensionResponse, StaticTableDataResponse, PostEventResponse
from tcsoa.gen.BusinessObjects import Fnd0StaticTable
from typing import List
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def getStaticTableData(cls, staticTable: Fnd0StaticTable) -> StaticTableDataResponse:
        """
        Returns a list of objects of type TableProperties which are associated with Fnd0StaticTable  object.
        Fnd0StaticTable object has an attribute fnd0StaticTableData which is an array of TableProperties objects.  Any
        failures will be returned with the input object mapped to the error message in the ServiceData list of partial
        errors.
        
        Use cases:
        This operation is used to get the data for attribute fnd0StaticTableData of Fnd0StaticTable object. Attribute
        fhd0StaticTableData is an array of TableProperties objects. When user selects Cdm0DataReqItemRevision object,
        the attribute cdm0EventsList is displayed in the summary as well as on View/Edit Properties menu in RAC. The
        attribute cdm0EventsList is Typed Reference to Fnd0StaticTable object.
        """
        return cls.execute_soa_method(
            method_name='getStaticTableData',
            library='Core',
            service_date='2010_09',
            service_name='DataManagement',
            params={'staticTable': staticTable},
            response_cls=StaticTableDataResponse,
        )

    @classmethod
    def postEvent(cls, input: List[PostEventObjectProperties], eventTypeName: str) -> PostEventResponse:
        """
        This operation will post an event for each of the Teamcenter business objects in the input list, with all the
        supplied information: 'secondaryObject', properties to be logged, and the error details. . Partial failures
        will be returned in the 'serviceData'.
        
        Use cases:
        Most events are posted by Teamcenter server logic. Use this operation to make an event known only to your
        client code recorded in Audit Manager or supported by Subscription Manager.
        Use Case1: Auditing events
        This operation helps auditing Teamcenter objects history by logging audit records when event eventTypeName
        occurs on primaryObject.
        - When site preference TC_audit_manager is set to ON and no Audit Definition exists for object type
        primaryObject and the eventTypeName, no audit records will be logged. Audit Definitions are Audit Manager
        Application configurations and can be viewed in Audit Manager Application.
        - When site preference TC_audit_manager is set to ON and Audit Definition exists for object type primaryObject
        and the eventTypeName, audit records will be logged with all the information provided in the structure
        PostEventObjectProperties
        - No audit records are written when preference TC_audit_manager is set to OFF or if the event posted is not
        defined as Auditable.
        
        
        
        Use Case2: Subscription Notifications    
        the site preference TC_subscription is set to ON , users can create subscriptions for notifications for certain
        events on Teamcenter Objects  The event posted must be described as subscribable and there should also exist an
        associated subscription object for the notification to occur.
        
        Exceptions:
        >Service Exception.
        Throws ServiceException with error message, error code, error count and severity.  Error code and message will
        be returned as per which audit or notification operation failed during processing.
        """
        return cls.execute_soa_method(
            method_name='postEvent',
            library='Core',
            service_date='2010_09',
            service_name='DataManagement',
            params={'input': input, 'eventTypeName': eventTypeName},
            response_cls=PostEventResponse,
        )

    @classmethod
    def setProperties(cls, info: List[PropInfo], options: List[str]) -> SetPropertyResponse:
        """
        This operation is provided to update Teamcenter object instances for the given name/value pairs. This operation
        works for all supported property value types. Each object need to be passed with its property name/value
        pairs.Passing options are not mandatory, empty list is allowed. When no options are provided, it just updates
        the objects as per the inputs. Alternatively you can pass following valid options to control updating the data.
        - QUERY: option is used to define the overall behavior of object properties setting from Excel Live and Word
        Live. Once this option is passed, server honours the preference value of TC_setProperties. Please see the
        Preferences and Environment Variables Reference documentation for preference TC_setProperties for more
        information.
        
        
        Note:It must be the 0th element when set as in the option list.
        - ENABLE_PSE_BULLETIN_BOARD: To enable the generation of PSE bulletin board events. These events are processed
        through Bulletin board callback mechanism. 
        
        """
        return cls.execute_soa_method(
            method_name='setProperties',
            library='Core',
            service_date='2010_09',
            service_name='DataManagement',
            params={'info': info, 'options': options},
            response_cls=SetPropertyResponse,
        )

    @classmethod
    def verifyExtension(cls, extensionInfo: List[VerifyExtensionInfo]) -> VerifyExtensionResponse:
        """
        This operation checks if an extension exists on an operation of a specific type.
        """
        return cls.execute_soa_method(
            method_name='verifyExtension',
            library='Core',
            service_date='2010_09',
            service_name='DataManagement',
            params={'extensionInfo': extensionInfo},
            response_cls=VerifyExtensionResponse,
        )

    @classmethod
    def createOrUpdateStaticTableData(cls, staticTableInfo: StaticTableInfo, rowProperties: List[RowData]) -> CreateOrUpdateStaticTableDataResponse:
        """
        This creates a new Table along with Rows or updates an existing Table with rows and their values based on input
        StaticTableInfo and created Table rows are added to the Table. ServiceData is updated with newly
        created/updated Table.
        
        Use cases:
        This operation is used to create/update the data for TableProperties objects of Fnd0StaticTable object. When
        user selects Cdm0DataReqItemRevision object, the attribute cdm0EventsList is displayed in the summary as well
        as on View/Edit Properties menu in RAC. The attribute cdm0EventsList is type referenced to Fnd0StaticTable.
        User can add the data in columns for each row of the table or adds rows to the table or deletes rows. After
        creation/updation of the table, user saves the object which invokes this SOA operation.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateStaticTableData',
            library='Core',
            service_date='2010_09',
            service_name='DataManagement',
            params={'staticTableInfo': staticTableInfo, 'rowProperties': rowProperties},
            response_cls=CreateOrUpdateStaticTableDataResponse,
        )

    @classmethod
    def getEventTypes(cls, input: List[EventObject]) -> EventTypesResponse:
        """
        The getEventTypes operation retrieves the valid Auditable and Subscribable events for each of the
        businessObject in the input 'EventObject' vector. When an event is auditable, you can audit actions on
        Teamcenter objects when that event happens on the businessObject. When an event is Subscribable, that means
        subscriptions can be created for that event. Partial failures, if any, will be returned in the serviceData.
        """
        return cls.execute_soa_method(
            method_name='getEventTypes',
            library='Core',
            service_date='2010_09',
            service_name='DataManagement',
            params={'input': input},
            response_cls=EventTypesResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Workflow._2007_06.Workflow import SetReleaseStatusResponse, ReleaseStatusInput
from tcsoa.utils import TcService


class WorkflowService(TcService):

    @classmethod
    def setReleaseStatus(cls, input: List[ReleaseStatusInput]) -> SetReleaseStatusResponse:
        """
        Manages the release status status of an object
        
        The permitted operations are Append, Delete, Rename and Replace
        Currently Append and Delete are supported
        With the delete operation if an empty string is passed in instead of the status name all
        statuses will be cleared for that set of workspace objects
        
        Exceptions:
        >Teamcenter::Soa::Server::ServiceException:
        """
        return cls.execute_soa_method(
            method_name='setReleaseStatus',
            library='Workflow',
            service_date='2007_06',
            service_name='Workflow',
            params={'input': input},
            response_cls=SetReleaseStatusResponse,
        )



from __future__ import annotations

from enum import Enum
from tcsoa.gen.BusinessObjects import ItemRevision, POM_object, Fnd0MarkupChange
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExpandOneLevelSearchScopeInput(TcBaseObj):
    """
    This operation expands the given parent product structure node to fetch its immediate children. A call to this
    operation is made when defining a search scope for impacted object search.
    
    :var operation: Type of mass update operation being performed. 
    Following are the valid values:
    129 - Replace Realization Source
    
    :var parentNode: Node in a product structure to be expanded.
    :var type: Type of child node
    :var revRule: Revision rule using which the expanded structure is to be configured with.
    """
    operation: int = 0
    parentNode: POM_object = None
    type: POM_object = None
    revRule: str = ''


@dataclass
class ExpandOneLevelSearchScopeOutput(TcBaseObj):
    """
    This is an output structure containing information about child nodes for a given parent node
    
    :var parentNode: Parent node of a product structure for which list of child nodes are requested.
    :var childNodes: List of child nodes for a given input parent node.
    """
    parentNode: POM_object = None
    childNodes: List[POM_object] = ()


@dataclass
class ExpandOneLevelSearchScopeResponse(TcBaseObj):
    """
    This is a structure containing information about child nodes for a given input parent node and partial errors
    occured during an operation.
    
    :var output: List of structure containing information about list of child nodes for a given input parent node of a
    product structure.
    :var serviceData: Standard service data to handle partial errors.
    """
    output: List[ExpandOneLevelSearchScopeOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetMarkupChangesForUpdateOutput(TcBaseObj):
    """
    Structure containing information about Fnd0MarkupChange objects per input ChangeItemRevision.
    
    :var changeItemRev: ChangeItemRevision whose markup changes are to be executed for mass update operation.
    :var markupChangeUIDs: List of unprocessed and/or previously failed Fnd0MarkupChange UIDs referenced by Fnd0Markup
    object associated with changeItemRev
    """
    changeItemRev: ItemRevision = None
    markupChangeUIDs: List[str] = ()


@dataclass
class GetMarkupChangesForUpdateResponse(TcBaseObj):
    """
    Response structure containing information about Fnd0MarkupChange objects per input ChangeItemRevision and standard
    service data for partial errors.
    
    
    :var output: List of structure containing information about Fnd0MarkupChange objects per input ChangeItemRevision.
    :var serviceData: Standard service data for partial errors.
    """
    output: List[GetMarkupChangesForUpdateOutput] = ()
    serviceData: ServiceData = None


@dataclass
class GetRevisionRulesResponse(TcBaseObj):
    """
    This operation returns the list of valid revision rules applicable for specific mass update operation type.
    
    :var revRuleNames: List of valid revision rules applicable for mass update operation type.
    :var serviceData: Standard service data for partial errors.
    """
    revRuleNames: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class ImpactedObjectDetailsInput(TcBaseObj):
    """
    This is a structure containing information required to get mass update specific details
    
    :var operation: Mass update operation type being performed. 
    Following are the valid values:
    129 - Replace Realization Source
    
    :var impactedObjectUIDs: This is a list of impacted object UIDs for which mass update specific details are required.
    :var processedImpactedUIDs: This is an optional list of impacted object UIDs for which mass update specific details
    were already fetched previously. Objects corresponding to these UIDs will be unloaded from the server since they
    are no longer needed.
    :var change: Change ItemRevision to record mass update results.
    """
    operation: int = 0
    impactedObjectUIDs: List[str] = ()
    processedImpactedUIDs: List[str] = ()
    change: ItemRevision = None


@dataclass
class ImpactedObjectDetailsOutput(TcBaseObj):
    """
    This is a structure containing information like whether the impacted object is selectable for performing an update,
    if not selectable then the reason for it being non-selectable, whether the impacted object is out of date etc.
    
    :var impactedObject: This is an impacted object in respect of which mass update specific details are fetched.
    :var isSelectable: Indicates whether the impacted object is selectable for update.
    :var isOutOfDate: Indicates whether the impacted object is out of date
    :var nonSelectableReason: This text informs why the impacted object is not selectable. For selectable impacted
    objects, this text is empty.
    """
    impactedObject: POM_object = None
    isSelectable: bool = False
    isOutOfDate: bool = False
    nonSelectableReason: str = ''


@dataclass
class ImpactedObjectDetailsResponse(TcBaseObj):
    """
    This is a structure containing mass update specific impacted object information and service data to hold partial
    errors.
    
    :var output: List of structure containing mass update specific impacted object information.
    :var serviceData: Standard service data to handle partial errors
    """
    output: List[ImpactedObjectDetailsOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ImpactedObjectStatusOutput(TcBaseObj):
    """
    This is a structure containing information about impacted object Save/Remove operation status and the text
    information describing the reason for the failure if any.
    
    :var impactedObject: This is an impacted object added to or removed from change ItemRevision
    :var status: Success/failure status for Save/Remove operation
    :var statusComment: This text informs why the operation on impacted object has failed. If the operation is
    successful then this text is empty.
    """
    impactedObject: POM_object = None
    status: bool = False
    statusComment: str = ''


@dataclass
class ImpactedObjectsQueryInput(TcBaseObj):
    """
    This is a structure containing information about inputs required to query impacted objects.
    
    :var operation: Mass update operation type being performed. This is used to identify type of filtering to use when
    searching for impacted objects. 
    Following are the valid values:
    129 - Replace Realization Source
    
    :var target: Target ItemRevision whose impacted objects are to be searched.
    :var change: Change ItemRevision to record mass update results.
    :var searchScope: Search scope for searching the impacted objects. If no search scope is provided, impacted object
    search is performed in all available product structures.
    """
    operation: int = 0
    target: ItemRevision = None
    change: ItemRevision = None
    searchScope: SearchScope = None


@dataclass
class ImpactedObjectsQueryOutput(TcBaseObj):
    """
    This is an output structure containing information about list of impacted UIDs per input target ItemRevision.
    
    :var target: Target ItemRevision whose impacted objects are queried for performing mass update operation.
    :var impactedObjectUIDs: List of all impacted object UIDs for a given target ItemRevision.
    """
    target: ItemRevision = None
    impactedObjectUIDs: List[str] = ()


@dataclass
class ImpactedObjectsQueryResponse(TcBaseObj):
    """
    This is a structure containing information about impacted object UIDs per target ItemRevision and service data for
    partial errors.
    
    :var output: List of structure containing impacted object UIDs information.
    :var serviceData: Standard service data to handle partial errors.
    """
    output: List[ImpactedObjectsQueryOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ManageImpactedObjectUpdateInput(TcBaseObj):
    """
    This is a structure containing information data required to manage impacted objects using markup 
    
    :var operation: Mass update operation type being executed
    Following are the valid values:
    129 - Replace Realization Source
    
    :var executionMode: This defines the change ItemRevision specific mass update operation being performed. Following
    are the valid values
    murAddImpactedObjectToMarkup
    murRemoveImpactedObjectFromMarkup
    :var target: Target ItemRevision whose impacted objects are to be searched for mass update
    :var replacement: The new replacement ItemRevision
    :var change: The change ItemRevision object to record mass update results in
    :var addToProblem: Whether to add the target ItemRevision to the problem folder 
    :var addToSolution: Whether to add the target ItemRevision to solution folder
    
    :var markupChange: The markup change correponding to an impacted object to be removed from the markup object and
    the impacted folder of change ItemRevision object
    :var impactedObjects: A list of impacted objects that needs to be managed on change ItemRevision
    """
    operation: int = 0
    executionMode: ManageUpdateOperationType = None
    target: ItemRevision = None
    replacement: ItemRevision = None
    change: ItemRevision = None
    addToProblem: bool = False
    addToSolution: bool = False
    markupChange: Fnd0MarkupChange = None
    impactedObjects: List[POM_object] = ()


@dataclass
class ManageImpactedObjectUpdatesResponse(TcBaseObj):
    """
    This is a structure containing status information about impacted object manage update operations.
    
    :var output: List of structure containing information about impacted object Save/Remove operation status
    :var serviceData: Standard service data for partial errors
    """
    output: List[ImpactedObjectStatusOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ProductScope(TcBaseObj):
    """
    This is a structure containing information about product level search scope.
    
    :var product: Top level product node to be included in the impacted object search scope.
    :var childScope: List of structure containing child scope information for searching the impacted objects.
    :var revRule: Revision rule to configure the impacted object search result.
    """
    product: POM_object = None
    childScope: List[ChildScope] = ()
    revRule: str = ''


@dataclass
class SearchScope(TcBaseObj):
    """
    This is a structure containing information about search scope for searching impacted objects.
    
    :var productScope: List of structure containing product scope information for searching the impacted objects.
    :var propNames: List of impacted object property names based on which the impacted objects are filtered.
    :var propValues: List of impacted object property values based on which the impacted objects are filtered.
    :var defaultRevRule: Default Revision rule to be applied on resultant impacted objects.
    """
    productScope: List[ProductScope] = ()
    propNames: List[str] = ()
    propValues: List[str] = ()
    defaultRevRule: str = ''


@dataclass
class UpdateImpactedObjectEndResponse(TcBaseObj):
    """
    Contains information about whether the cleanup activity is performed successfully or not.
    
    :var isCleanupSuccessful: A list of flags indicating whether the cleanup activity is performed succesfully for each
    input.
    :var serviceData: Standard service data for partial errors.
    """
    isCleanupSuccessful: List[bool] = ()
    serviceData: ServiceData = None


@dataclass
class UpdateImpactedObjectInput(TcBaseObj):
    """
    This is a structure containing information required to update impacted object for mass update operation.
    
    :var operation: Mass update operation type being executed. 
    Following are the valid values:
    129 - Replace Realization Source
    
    :var target: Target ItemRevision whose impacted objects are to be searched for mass update
    :var replacement: The new replacement ItemRevision
    :var change: The change ItemRevision object to record mass update results in
    :var addToProblem: Add the target ItemRevision to the problem folder
    :var addToSolution: Add the replacement ItemRevision to solution folder
    :var impactedObjects: A list of impacted objects to be updated
    :var isSelected: A list of boolean values that indicates whether the impacted objects are selected for mass update
    operation
    """
    operation: int = 0
    target: ItemRevision = None
    replacement: ItemRevision = None
    change: ItemRevision = None
    addToProblem: bool = False
    addToSolution: bool = False
    impactedObjects: List[POM_object] = ()
    isSelected: List[bool] = ()


@dataclass
class UpdateImpactedObjectResponse(TcBaseObj):
    """
    This is a structure containing information about impacted object update status for every object selected for mass
    update operation. It also contains partial error information in ServiceData.
    
    :var output: List of structure containing information about impacted object update operation status.
    :var serviceData: Standard service data for partial errors.
    :var updateRequestId: Unique Identifier per update request initiated from the 'Mass Update Realization' wizard. If
    the update is being performed in batches then this Identifier is same for all the batches. 
    """
    output: List[ImpactedObjectStatusOutput] = ()
    serviceData: ServiceData = None
    updateRequestId: str = ''


@dataclass
class UpdateImpactedObjectStartResponse(TcBaseObj):
    """
    Response structure containing update request identifier and standard service data for partial errors.
    
    
    :var updateRequestId: Unique Update Request Identifier per update request initiated by user while performing 'Mass
    Update Realization'. If the update is being performed in batches then this Identifier is same for all the batches.
    :var serviceData: Standard service data for partial errors.
    
    Note: Member 'serviceData' available as part of this structure is not being used presently. This member has been
    added to this structure for Future use case.
    """
    updateRequestId: str = ''
    serviceData: ServiceData = None


@dataclass
class ValidateChangeObjectForMassUpdateOutput(TcBaseObj):
    """
    This is a structure containing information whether the input ChangeItemRevision is valid for Mass Update type.
    
    :var changeItemRev: Input ChangeItemRevision.
    :var isValid: This flag is set to true if the corresponding input Change Item revision is valid for Mass Update
    type.
    -If API is called with input massUpdateType="massUpdate", this flag is set to false if Fnd0MarkupChange object
    associated with input ChangeItemRevision is found with mass update operation value = 129
    -If API is called with input massUpdateType="massUpdateRealization", this flag is set to false if Fnd0MarkupChange
    object associated with input ChangeItemRevision is found with mass update operation value != 129
    """
    changeItemRev: ItemRevision = None
    isValid: bool = False


@dataclass
class ValidateChangeObjectForMassUpdateResponse(TcBaseObj):
    """
    This is a structure that contains the information whether the input ChangeItemRevision is valid for Mass Update
    type. It also holds the standard service data for partial errors.
    
    :var output: List of structure containing information whether the input ChangeItemRevision is valid for Mass Update
    operation.
    :var serviceData: Standard service data for partial errors.
    """
    output: List[ValidateChangeObjectForMassUpdateOutput] = ()
    serviceData: ServiceData = None


@dataclass
class ChildScope(TcBaseObj):
    """
    This is a structure containing information about a child node and a boolean flag indicating whether this child node
    has to be included recursively in the search scope of impacted objects.
    
    :var child: Child node to be included in the impacted object search scope.
    :var recurse: Indicates whether the child nodes are to be included recursively for the impacted object search scope.
    """
    child: POM_object = None
    recurse: bool = False


@dataclass
class ExecuteMarkupChangeInput(TcBaseObj):
    """
    Input structure containing information about Change Item revision and it's Fnd0MarkupChange object UIDs to be
    executed.
    
    :var changeItemRev: ChangeItemRevision whose associated Fnd0MarkupChange objects are to be executed.
    :var markupChangeUIDs: List of Fnd0MarkupChange object UIDs associated with ChangeItemRevision
    """
    changeItemRev: ItemRevision = None
    markupChangeUIDs: List[str] = ()


@dataclass
class ExecuteMarkupChangeOutput(TcBaseObj):
    """
    This is a structure containing information about an update status and the list of updated Fnd0MarkupChange objects
    per input ChangeItemRevision.
    
    :var changeItemRev: ChangeItemRevision whose associated Fnd0MarkupChange objects are executed
    :var markupChangeObjs: A list of Fnd0MarkupChange objects that are executed
    :var isUpdateSuccessful: This flag is set to true only if all the input Fnd0MarkupChange objects executes
    succesfully.
    """
    changeItemRev: ItemRevision = None
    markupChangeObjs: List[Fnd0MarkupChange] = ()
    isUpdateSuccessful: bool = False


@dataclass
class ExecuteMarkupChangeResponse(TcBaseObj):
    """
    This is a response structure containing update status information about Fnd0MarkupChange objects per input
    ChangeItemRevision and standard service data for partial errors.
    
    :var output: List of structure containing update status and updated Fnd0MarkupChange objects per input
    ChangeItemRevision.
    
    :var serviceData: Standard service data for partial errors.
    """
    output: List[ExecuteMarkupChangeOutput] = ()
    serviceData: ServiceData = None


class ManageUpdateOperationType(Enum):
    """
    Defines enum values for manage update operation types
    """
    murAddImpactedObjectToMarkup = 'murAddImpactedObjectToMarkup'
    murRemoveImpactedObjectFromMarkup = 'murRemoveImpactedObjectFromMarkup'



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Awp0AdvancedSearchInput, Awp0AdvancedQueryCriteria
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AdvancedQueryCriteriaResponse(TcBaseObj):
    """
    Response structure for selected Query Criteria.
    
    :var serviceData: The Service Data.
    :var advancedQueryCriteria: The newly created Advanced Query Criteria business object.
    """
    serviceData: ServiceData = None
    advancedQueryCriteria: Awp0AdvancedQueryCriteria = None


@dataclass
class AdvancedSearchResponse(TcBaseObj):
    """
    Response structure for AdvancedSearch service.
    
    :var advancedSearchInput: The newly created AdvancedSearch Input business object.
    :var serviceData: The Service Data.
    """
    advancedSearchInput: Awp0AdvancedSearchInput = None
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.Manufacturing._2020_01.DataManagement import AssociateOrRemoveScopeInput
from typing import List
from tcsoa.gen.Internal.Manufacturing._2020_01.StructureSearch import SearchScopedStructureInputInfo, SearchScopedStructureResponse
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class StructureSearchService(TcService):

    @classmethod
    def searchScopedStructure(cls, searchScopedStructureInput: SearchScopedStructureInputInfo) -> SearchScopedStructureResponse:
        """
        This service operation searches for objects in a BOP structure based on the input search criteria. It refers to
        related scope in Bill Of Material structure to obtain the final results. The search criteria consists of object
        type and query type.
        
        Use cases:
        Use Case 1 : A user opens a BOP structure. User selects a process line object from BOP structure and selects
        query "Suggested Product". This query identifies assigned products in process line from BOP structure and then
        calculates unassigned products using scope from BOM structure.
        Use Case 2 : A user opens a BOP structure. User selects a process line object from BOP structure and selects
        query "Suggested Discrete Features". This query Identifies assigned weldpoints in process line, find out the
        parts connected to Welds, which are NOT assigned in the process.
        Use Case 3 : A user opens a BOP structure. User selects a process line object from BOP structure and selects
        query "Suggested Datum Points". This query identifies assigned datumpoints in process line, find out the parts
        connected to datums, which are present in the scope from product.
        """
        return cls.execute_soa_method(
            method_name='searchScopedStructure',
            library='Internal-Manufacturing',
            service_date='2020_01',
            service_name='StructureSearch',
            params={'searchScopedStructureInput': searchScopedStructureInput},
            response_cls=SearchScopedStructureResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def associateOrRemoveScopesForProcess(cls, associateOrRemoveInput: List[AssociateOrRemoveScopeInput]) -> ServiceData:
        """
        This service operation associates the process line object from the BOP structure to either the BOMLine object
        scope or to Mfg0BvrWorkarea object from Bill of Equipment (BOE) structure. It creates Fnd0ProcessScopeRel
        relation between the item revision of process line object and the AbsOccurrence of BOMLine object or
        Mfg0BvrWorkarea object. This operation also removes the already defined association.
        
        Use cases:
        Use Case 1 : A user opens a BOM and a BOP structure. User selects a process line object from BOP structure,
        select one or more objects from BOM structure and associate the BOMLine objects to process line object. A
        Fnd0ProcessScopeRel is created between the item revision of process line object to AbsOccurrence of  selected
        scope line objects.
        Use Case 2 : A user opens a BOE and a BOP structure. User selects a process line object from BOP structure,
        select one or more objects from BOE structure and associate the Mfg0BvrWorkarea objects to process line object.
        A Fnd0ProcessScopeRel is created between the item revision of process line object to AbsOccurrence of  selected
        scope line objects.
        Use Case 3 : A user opens a BOP  structure. From the list of already defined scope lines from BOM or BOE
        structure, user select one or many line objects. The selected objects are disassociated from the process line
        object.
        """
        return cls.execute_soa_method(
            method_name='associateOrRemoveScopesForProcess',
            library='Internal-Manufacturing',
            service_date='2020_01',
            service_name='DataManagement',
            params={'associateOrRemoveInput': associateOrRemoveInput},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Validation._2008_06.Validation import EvaluateValidationResultsWithRulesInfo, ValidationParserResponse, ValidationExpressionInfo, EvaluateValidationResultsWithRulesResponse
from typing import List
from tcsoa.utils import TcService


class ValidationService(TcService):

    @classmethod
    def parseRequirementExpressions(cls, inputs: List[ValidationExpressionInfo]) -> ValidationParserResponse:
        """
        This operation checks the syntax of a requirement formula string and parses it into a list of variables and
        functions if the string has no syntax error.
        
        Use cases:
        The property requirement_formula on ValReqRevision object need to follow a specific syntax and the variable
        names in the property requirement_formula need to match with the objects in property requirement_expressions on
        ValReqRevision object.
        This operation can be used for following use cases:
        - Checks the syntax of a requirement formula string before setting it as the value of property
        requirement_formula on ValReqRevision object
        - Extract a list of variable names from a requirement formula string and use these names to populate the
        property requirement_expressions on ValReqRevision object.
        
        """
        return cls.execute_soa_method(
            method_name='parseRequirementExpressions',
            library='Internal-Validation',
            service_date='2008_06',
            service_name='Validation',
            params={'inputs': inputs},
            response_cls=ValidationParserResponse,
        )

    @classmethod
    def evaluateValidationResultsWithRules(cls, inputs: List[EvaluateValidationResultsWithRulesInfo]) -> EvaluateValidationResultsWithRulesResponse:
        """
        This operation evaluates a set of objects against a validation rule set or a set of validation checkers to
        verify whether the validation results on these objects satisfy the validation rule set or checkers.  
        The objects can be any combination of the following WorkspaceObject types: Folder, Item, ItemRevision and
        Dataset. The checking criteria can be either a validation rule set item revision or a set of validation
        checkers and the applicable dataset types.
        This operation will first find the validation target datasets from the input object list, and then evaluate all
        results on these target dataset against the validation rule set or checkers. This operation only supports the
        validation results for NX Agents, such as NX Check Mate and NX RDDV.
        
        Use cases:
        In some companies, to ensure the CAD model quality, all NX parts must pass certain NX Check Mate checks before
        it can be released or a new baseline can be created.  There are several existing workflow handlers and a
        baseline operation can be used check the existing validation results on the to be released or to be baselined
        item revisions against a validation rule set and indicate whether the item revision pass the rule set  or not.
        In the rule evaluation tab of Validation Results Viewer in the rich client, a user can also select a list of
        objects and a validation rule set to see the evaluation results.
        This operation provides a programmatic way to perform the similar evaluation as in above mentioned workflow
        handlers or rule evaluation capability in the rich client.
        In addition, this operation can also check the validation results against a set of checkers and dataset types
        if no validation rule set are provided.
        """
        return cls.execute_soa_method(
            method_name='evaluateValidationResultsWithRules',
            library='Internal-Validation',
            service_date='2008_06',
            service_name='Validation',
            params={'inputs': inputs},
            response_cls=EvaluateValidationResultsWithRulesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject
from tcsoa.gen.Internal.IcsAw._2018_05.Classification import SaveClassificationObjectsResponse, SearchCriteria, FindClassificationInfoResponse, ClassificationInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ClassificationService(TcService):

    @classmethod
    def deleteClassificationObjects(cls, classificationObjects: List[BusinessObject]) -> ServiceData:
        """
        Deletes one or more traditional or Classification Standard Taxonomy (CST) classification objects permanently.
        The classified workspace object associated with the classification object will not be deleted. When deleting a
        classification object, additional structures related to this classification object are deleted as well. These
        are the icm0 and Cst0PropertyRecord objects, Cls0ClassifiedBy object, and Ptn0Membership object.
        """
        return cls.execute_soa_method(
            method_name='deleteClassificationObjects',
            library='Internal-IcsAw',
            service_date='2018_05',
            service_name='Classification',
            params={'classificationObjects': classificationObjects},
            response_cls=ServiceData,
        )

    @classmethod
    def findClassificationInfo(cls, workspaceObjects: List[WorkspaceObject], searchCriteria: List[SearchCriteria], classificationDataOptions: int) -> FindClassificationInfoResponse:
        """
        Finds traditional and Classification Standard Taxonomy (CST) classification information based on the input
        criteria.
        
        If a list of WorkspaceObject objects are provided; then this operation finds all the classification objects for
        the given WorkspaceObject.
        If a given WorkspaceObject is not classified this operation finds and returns standalone classification object;
        which could then be linked to given WorkspaceObject.
        If a given workspace object is not classified and also system does not contain standalone classification object
        which could be linked to the WorkspaceObject then this operation will return classification class hierarchy
        (For traditional classification, children of "Classification Root" (ICM). For CST classification, children of
        the Default Hierarchy View) which could be used for browsing and classifying an object in a classification
        class of interest.
        
        If a search criteria is provided; then this operation searches for classification classes in classification
        hierarchy and returns their metadata information like class ID, name, class parents etc. If both
        'searchCriterias' and 'workspaceObjects' are provided, 'workspaceObjects' list is ignored and search criteria
        is used.
        
        Use cases:
        This operation could be used when user needs to find classification objects, for both traditional and
        Classification Standard Taxonomy (CST), based on workspace objects. Each time a WorkspaceObject is classified
        in a classification class a classification object is created. After searching for all the classification
        objects corresponding to a WorkspaceObject, user can find more information about the classification(s) of a
        WorkspaceObject. 
        The operation 'findClassificationInfo' can be used to get detailed information about the classification
        objects, classification classes where these classification objects belong. After getting information about
        classification objects the user can choose to modify or delete the objects by using operation:
        'saveClassificationObjects' or 'deleteClassificationObjects', both of which support traditional and
        Classification Standard Taxonomy (CST) classes.
        
        The operation 'findClassificationInfo' can also be used when a user wants to search for classes in
        classification hierarchy by using any of class' properties. After searching for classification classes; the
        user can use that information to browse through a subset of classification hierarchy and classify a
        classifiable workspace object in a classification class of interest.
        """
        return cls.execute_soa_method(
            method_name='findClassificationInfo',
            library='Internal-IcsAw',
            service_date='2018_05',
            service_name='Classification',
            params={'workspaceObjects': workspaceObjects, 'searchCriteria': searchCriteria, 'classificationDataOptions': classificationDataOptions},
            response_cls=FindClassificationInfoResponse,
        )

    @classmethod
    def saveClassificationObjects(cls, classificationObjects: List[ClassificationInfo], deltaUpdateFlag: int) -> SaveClassificationObjectsResponse:
        """
        Creates or updates one or more classification objects and (optionally) attach them to a WorkspaceObject object,
        thus classifying it. When the Classification objects are not associated with any WorkspaceObject they act as
        standalone classification objects. A classification object is also called an ICO object. This supports
        traditional ICO, next generation Cls0Object, and Classification Standard Taxonomy (CST) Cls0CstObject.
        Additionally, this operation can perform a delta update for Cls0CstObject object.
        
        Use cases:
        User wants to classify a workspace object or create a standalone classification object (ICO) or update an
        existing classification object in Active Workspace, in either a traditional or Classification Standard Taxonomy
        (CST) class. Additionally, the user could be performing a delta update on a CST Property Record. This operation
        can be combined with other operations like createItems() to create workspace object and then associate the
        workspace object to the classification object. Before creating a classification object, a classification class
        hierarchy should already be created by the classification admin user in Teamcenter. This hierarchy should
        include a storage class (a class that allows instances to be created and associated to it) for which the
        classification objects need to be created. Values of any attributes associated with classification objects can
        also be populated.
        """
        return cls.execute_soa_method(
            method_name='saveClassificationObjects',
            library='Internal-IcsAw',
            service_date='2018_05',
            service_name='Classification',
            params={'classificationObjects': classificationObjects, 'deltaUpdateFlag': deltaUpdateFlag},
            response_cls=SaveClassificationObjectsResponse,
        )



from __future__ import annotations

from tcsoa.gen.RelationshipViewer._2012_10.NetworkEngine import QueryNetworkInputs, NetworkResponse, CreateRelationsResponse, GraphTypeListResponse, CreateRelationInput, GraphParamMap, GraphStyleDefResponse
from typing import List
from tcsoa.utils import TcService


class NetworkEngineService(TcService):

    @classmethod
    def createRelations(cls, inputs: List[CreateRelationInput]) -> CreateRelationsResponse:
        """
        The operation will create the specified relation between the input objects (primary and secondary objects) for
        each input structure.
        
        Use cases:
        You can create relations that can be represented as edges in network graph. Such as GRM (General Relation
        Management) relation, BOM structure and PS Connection.
        """
        return cls.execute_soa_method(
            method_name='createRelations',
            library='RelationshipViewer',
            service_date='2012_10',
            service_name='NetworkEngine',
            params={'inputs': inputs},
            response_cls=CreateRelationsResponse,
        )

    @classmethod
    def getGraphStyleDef(cls) -> GraphStyleDefResponse:
        """
        Provides a style definition which can be used for client to render graph data. For example, it returns style
        definition of node shape and color,edge type and color.
        """
        return cls.execute_soa_method(
            method_name='getGraphStyleDef',
            library='RelationshipViewer',
            service_date='2012_10',
            service_name='NetworkEngine',
            params={},
            response_cls=GraphStyleDefResponse,
        )

    @classmethod
    def getViews(cls) -> GraphTypeListResponse:
        """
        The RelationShipViewer library provides graph (nodes connected via edges) views of data elements. Each graph
        has a defined view type. The view type is a container for the rules that determine what should be included in a
        graph and how it should be represented. The choice of graph view is determined based on business need. 
        
        This API provides a list of the available graph view types. A graph view is a set of configuration that can be
        used for network expansion. For example: view name, graph parameters, inquires, graph presentation rules. The
        view list is role based, different role may get different view list.
        """
        return cls.execute_soa_method(
            method_name='getViews',
            library='RelationshipViewer',
            service_date='2012_10',
            service_name='NetworkEngine',
            params={},
            response_cls=GraphTypeListResponse,
        )

    @classmethod
    def getViews2(cls, key: str) -> GraphTypeListResponse:
        """
        This operation provides a list of the available graph view types identified by the input key. A graph view is a
        set of configuration that can be used for network expansion. For example: view name, graph parameters,
        inquires, graph presentation rules. The view list is role based, different role may get different view list.
        
        The RelationShipViewer library provides graph (nodes connected via edges) views of data elements. Each graph
        has a defined view type. The view type is a container for the rules that determine what should be included in a
        graph and how it should be represented. The choice of graph view is determined based on business need.
        """
        return cls.execute_soa_method(
            method_name='getViews2',
            library='RelationshipViewer',
            service_date='2012_10',
            service_name='NetworkEngine',
            params={'key': key},
            response_cls=GraphTypeListResponse,
        )

    @classmethod
    def queryNetwork(cls, rootIds: List[str], viewName: str, graphParamMap: GraphParamMap, inquiries: List[str]) -> NetworkResponse:
        """
        This API produces a graph of data corresponding to the input parameters. For Teamcenter data source, the graph
        nodes are Teamcenter business objects, the relation between the returns objects and the input root objects may
        be Teamcenter Relation class (GRM) relationships, TraceLink and its sub types, Where used, where referenced,
        BOM relation and Tag reference.
        """
        return cls.execute_soa_method(
            method_name='queryNetwork',
            library='RelationshipViewer',
            service_date='2012_10',
            service_name='NetworkEngine',
            params={'rootIds': rootIds, 'viewName': viewName, 'graphParamMap': graphParamMap, 'inquiries': inquiries},
            response_cls=NetworkResponse,
        )

    @classmethod
    def queryNetwork2(cls, input: QueryNetworkInputs) -> NetworkResponse:
        """
        This operation produces a graph of data corresponding to the input parameters. The graph nodes are Teamcenter
        business objects, the relation between the returned objects and the input root objects may be Teamcenter
        Relation class (GRM) relationships, TraceLink and its sub types, Where used, where referenced, BOM relation and
        Tag reference.
        You can query relations supported in Teamcenter for the input root Business Objects, such as GRM (General
        Relationship Management) relation, BOM structure, Connectility, etc.
        """
        return cls.execute_soa_method(
            method_name='queryNetwork2',
            library='RelationshipViewer',
            service_date='2012_10',
            service_name='NetworkEngine',
            params={'input': input},
            response_cls=NetworkResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExtensionMountAttachTypeDetails(TcBaseObj):
    """
    ExtensionMountAttachTypeDetails represents a list of MountAttachDetailTypeElement for each extension name.
    
    :var extensionName: Extension name that needs to be compared.
    An extension name is the ID of a "serverExtension" registered in Accountability Check&rsquo;s plugin.xml. It can be
    located in com.teamcenter.rac.cme.accountabilitycheck.relations package. 
    The server extension for physical connection is "Mfg0MountAttach_Relation".
    :var mountAttachDetailTypes: A list of details of each physical attachment type.
    """
    extensionName: str = ''
    mountAttachDetailTypes: List[MountAttachDetailTypeElement] = ()


@dataclass
class MountAttachComparisonsResponse(TcBaseObj):
    """
    MountAttachComparisonDetailsResponse represents a list of MountAttachTypeDetail element and ServiceData.
    
    :var mountAttachTypeDetails: A list of MountAttachTypeDetail elements - one for each input equivalent set.
    :var serviceData: Object that captures any partial errors.
    """
    mountAttachTypeDetails: List[MountAttachTypeDetail] = ()
    serviceData: ServiceData = None


@dataclass
class MountAttachDetail(TcBaseObj):
    """
    MountAttachDetail holds a set of BOE objects connected using physical attachment relations and match type
    information for each BOMLine in the input equivalent set.
    
    :var attachments: For each BOMLine in input equivalent set holds the physical attachment objects. The physical
    attachments can be both tool and weld gun objects.
    :var matchType: To indicate the type of match between physical attachments. Values are from set {0, 1, 2} which
    represents set {"Full Match", "Source Modified", "Target Modified"} respectively.
    """
    attachments: List[BusinessObject] = ()
    matchType: int = 0


@dataclass
class MountAttachDetailTypeElement(TcBaseObj):
    """
    MountAttachDetailTypeElement contains a list of all the physical attachments for a particular attachment relation
    type.
    
    :var relationType: The relation type of physical attachment (Mfg0MEPhysicalAttachment, Mfg0MEMountToolToRobot).
    :var mountAttachDetails: A list of BOE objects connected using physical attachment relations and match type
    information for each BOMLine in the input equivalent set.
    """
    relationType: BusinessObject = None
    mountAttachDetails: List[MountAttachDetail] = ()


@dataclass
class MountAttachTypeDetail(TcBaseObj):
    """
    MountAttachTypeDetail pairs the ExtensionMountAttachTypeDetails element and a list of equivalent BOMLine objects
    with an index.
    
    :var index: Index of the extension in the input equivalent set for which these details were calculated. It
    represents the order in which the extensions are provided in the input equivalent set.
    :var equivalentLines: A list of all equivalent lines in the input equivalent set (all equivalent sources in
    sequence and then all targets in sequence) of MEWorkarea objects for which attachments are to be compared.
    :var mountAttachTypeExtensions: A list of physical attachment type details of this equivalent set.
    """
    index: int = 0
    equivalentLines: List[BusinessObject] = ()
    mountAttachTypeExtensions: List[ExtensionMountAttachTypeDetails] = ()



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from tcsoa.gen.BusinessObjects import WorkspaceObject
from dataclasses import dataclass


@dataclass
class FailedObjInfo(TcBaseObj):
    """
    The failed client object property name and failed error list.
    
    :var failedForeignObjectName: the name of the failed client object property.
    :var errorStack: the list of failure errors.
    """
    failedForeignObjectName: str = ''
    errorStack: List[str] = ()


@dataclass
class AttrExchangeMapping(TcBaseObj):
    """
    The metadata exchange mapping information and the client object property name.
    
    :var foreignObjectPropertyName: the property name of the client object which the Teamcenter property would be
    exchanged with.
    :var mapping: the top level MappingObject.
    """
    foreignObjectPropertyName: str = ''
    mapping: MappingObject = None


@dataclass
class AttrExchangeMappingSetValue(TcBaseObj):
    """
    The metadata exchange mapping information and the value to be set on the Teamcenter property.
    
    :var attrExchange: the attribute exchange mapping information.
    :var tcPropertyValue: the property value to be on the Teamcenter object attribute.
    """
    attrExchange: AttrExchangeMapping = None
    tcPropertyValue: str = ''


@dataclass
class MappingObject(TcBaseObj):
    """
    This is the key structure holding the metadata exchange mapping information.  Only one of the three values can be
    set: nextLevelMappedObject, relationObject, or attributeName.  
    - If the next level mapping is linked by referenced Teamcenter object, then the nextLevelMappedObject is set.  
    - If the next level mapping is linked by Teamcenter relation, then the relationObject is set.  
    - If the current Teamcenter object is where the property is from, then the attributeName is set.
    
    
    
    :var objectType: the Teamcenter object type name.
    :var referencedName: the Teamcenter property on the upper level Teamcenter object by which the current Teamcenter
    object is being referenced (For example, an Item Revision relates to a Dataset with the References relation, the
    upper level object is the Item Revision, and the current object is the Dataset).  This value is set when Teamcenter
    property type is any other Teamcenter type.
    :var path: the indicator of how to find the next level.  If ReferencedObject is the value then use referencedObject
    from the MappingObject.  If Realtion is the value, then use relationObject, or if Leaf is the value then use
    attributeName.
    :var attributeName: the name of the Teamcenter property on the last level object for the exchange.
    :var nextLevelMappedObject: the MappingObject collection which contains mapping information for the next level. 
    This will be a vector of single object.  Using a collection since circular references are not supported.
    :var relationObject: the relationship information between the current Teamcenter object and the next level object.
    """
    objectType: str = ''
    referencedName: str = ''
    path: str = ''
    attributeName: str = ''
    nextLevelMappedObject: List[MappingObject] = ()
    relationObject: List[Relation] = ()


@dataclass
class Relation(TcBaseObj):
    """
    The information about relationship between two Teamcenter objects.
    
    :var relationName: the relationship name that exists between two Teamcenter objects.
    :var mappedObject: a next level object which contains mapping information.  This collection will only contain one
    object, using a collection since circular references are not supported.
    :var relationType: the information about what the relation is between two Teamcenter objects.  The values could be
    Primary or Secondary. If it is primary, the containing object is the primary object in the relation.  If it is
    secondary, the containing object is the secondary object in the relation.
    """
    relationName: str = ''
    mappedObject: List[MappingObject] = ()
    relationType: str = ''


@dataclass
class ResolveAndGetData(TcBaseObj):
    """
    The Teamcenter object property values based on the given metadata exchange mapping information.
    
    :var foreignObjectPropertyName: the property name of the client object which the Teamcenter object property would
    be exchanged with.
    :var tcAttrVal: the Teamcenter property value to be set on the client object property.
    :var isFailed: this value is true if the mapping resolve for this input object failed.
    :var errorStack: these are the errors return from the server for the particular mapping resolve or getting
    properties.
    """
    foreignObjectPropertyName: str = ''
    tcAttrVal: TcAttributeValue = None
    isFailed: bool = False
    errorStack: List[str] = ()


@dataclass
class ResolveAndGetResult(TcBaseObj):
    """
    The WorkspaceObject (the current implementation in the Microsoft Office client, this is the Dataset) object and its
    corresponding resolved attribute mapping outputs.
    
    :var dataset: the WorkspaceObject (the current implementation in the Microsoft Office client, this is the Dataset)
    object from which the resolving of the mapping begins.
    :var resolveAndGetDataList: the list of resolved attribute structures whether it succeeded or failed.
    """
    dataset: WorkspaceObject = None
    resolveAndGetDataList: List[ResolveAndGetData] = ()


@dataclass
class ResolveAndSetResult(TcBaseObj):
    """
    The Teamcenter object and the corresponding failed mapping information about which Teamcenter object property
    setting has failed.
    
    :var dataset: the WorkspaceObject (the current implementation in the Microsoft Office client, this is the Dataset) 
    object where the attribute mapping start from.
    :var failedObjInfos: the list of the failed mappping information.
    """
    dataset: WorkspaceObject = None
    failedObjInfos: List[FailedObjInfo] = ()


@dataclass
class ResolveAttrMappingsAndGetPropertiesInfo(TcBaseObj):
    """
    The input structure for ResolveAttrMappingsAndGetProperties service method.
    
    :var locale: the locale information for the dataset content.
    :var datsetObject: the WorkspaceObject (the current implementation in the Microsoft Office client, this is the
    Dataset) object from which the metadata exchange is initiated.
    :var attrExchange: the list of structures with the metadata exchange mapping information.
    """
    locale: str = ''
    datsetObject: WorkspaceObject = None
    attrExchange: List[AttrExchangeMapping] = ()


@dataclass
class ResolveAttrMappingsAndGetPropertiesResponse(TcBaseObj):
    """
    The return structure for resolving metadata exchange mappings and the Teamcenter object property values.
    
    :var resolveAndGetResults: the list of structures that have the Teamcenter object property values.
    :var serviceData: the list of errors.  The error would be associated with the property names of the client object
    property for the which the metadata exchange failed.
    """
    resolveAndGetResults: List[ResolveAndGetResult] = ()
    serviceData: ServiceData = None


@dataclass
class ResolveAttrMappingsAndSetPropertiesInfo(TcBaseObj):
    """
    The input structure for resolving metadata exchange mappings and to set the Teamcenter object property.
    
    :var locale: the locale information for the dataset content.
    :var datasetObject: the WorkspaceObject (normally is Dataset) object from which the metadata exchange is initiated.
    :var attrExchange: the list of structures with the metadata exchange mapping information.
    """
    locale: str = ''
    datasetObject: WorkspaceObject = None
    attrExchange: List[AttrExchangeMappingSetValue] = ()


@dataclass
class ResolveAttrMappingsAndSetPropertiesResponse(TcBaseObj):
    """
    The return structure for resolving metadata exchange mappings, and the Teamcenter object property values.
    
    :var serviceData: the list of partial errors returned for failed metadata exchange object.
    :var resolvedAndSetResults: the list of structures that have the client object property name that fails to set and
    the corresponding list of errors.
    """
    serviceData: ServiceData = None
    resolvedAndSetResults: List[ResolveAndSetResult] = ()


@dataclass
class TcAttributeValue(TcBaseObj):
    """
    The resolved attribute information.
    
    :var tcVal: the Teamcenter property value.  The actual value is converted to string before sending to the client.
    :var typeOfVal: the Teamcenter property type.  The type of the value can be of char, double, float, int, logical,
    short or string.  The client and the server will need to do the conversion based on these types.
    """
    tcVal: str = ''
    typeOfVal: str = ''



from __future__ import annotations

from tcsoa.gen.Administration._2007_01.UserManagement import CreateDisciplinesResponse, CreateDisciplinesIn
from typing import List
from tcsoa.utils import TcService


class UserManagementService(TcService):

    @classmethod
    def createDisciplines(cls, disciplines: List[CreateDisciplinesIn]) -> CreateDisciplinesResponse:
        """
        This operation creates a list of new Discipline objects based on the list of CreateDisciplineIn objects. If 
        parent group is specified in the CreateDisciplinesIn object, the discipline objects would be added into the
        group. This operation requires system administration or group administration privilege.
        """
        return cls.execute_soa_method(
            method_name='createDisciplines',
            library='Administration',
            service_date='2007_01',
            service_name='UserManagement',
            params={'disciplines': disciplines},
            response_cls=CreateDisciplinesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, TransferOptionSet
from tcsoa.gen.GlobalMultiSite._2008_06.ImportExport import ExportObjectsToOfflinePackageResponse, ImportObjectsFromOfflinePackageResponse
from typing import List
from tcsoa.gen.GlobalMultiSite._2007_06.ImportExport import NamesAndValue
from tcsoa.utils import TcService


class ImportExportService(TcService):

    @classmethod
    def importObjectsFromOfflinePackage(cls, fmsTicket: str, optionSetTag: TransferOptionSet, optionNamesAndValues: List[NamesAndValue], sessionOptionAndValues: List[NamesAndValue]) -> ImportObjectsFromOfflinePackageResponse:
        """
        This operation imports the data of a briefcase into Teamcenter. A packed briefcase contains a TC XML file which
        holds a number of Teamcenter objects and related physical dataset files. After import, those objects will be
        replica in the importing site.
        
        Use cases:
        In data exchange, user may transfer a briefcase file from the source site to a remote site. In the importing
        site, user can use this operation to import the briefcase file into Teamcenter. After import, the objects held
        in the TC XML file will be created or updated if they have been imported before, physical dataset files will
        uploaded and attached to the related datasets.
        """
        return cls.execute_soa_method(
            method_name='importObjectsFromOfflinePackage',
            library='GlobalMultiSite',
            service_date='2008_06',
            service_name='ImportExport',
            params={'fmsTicket': fmsTicket, 'optionSetTag': optionSetTag, 'optionNamesAndValues': optionNamesAndValues, 'sessionOptionAndValues': sessionOptionAndValues},
            response_cls=ImportObjectsFromOfflinePackageResponse,
        )

    @classmethod
    def exportObjectsToOfflinePackage(cls, reason: str, sites: List[BusinessObject], objects: List[BusinessObject], optionSetTag: TransferOptionSet, optionNamesAndValues: List[NamesAndValue], sessionOptionAndValues: List[NamesAndValue]) -> ExportObjectsToOfflinePackageResponse:
        """
        Exports the objects to an offline package called briefcase. This operation returns a structure which includes
        the briefcase's FMS file ticket and exporter log file's FMS ticket. The briefcase ticket is used for
        downloading the briefcase file from the server to the client side by using FMS utility. Exporter log ticket is
        used for downloading the exporter log. 
        The briefcase is a package contains an exported TC XML file and a set of physical dataset files. The TC XML
        file holds the exported objects traversed by TC XML Export framework with the input TransferOptionSet and
        options, session options.
        Exporter log include the exporting status of the related objects. 
        
        
        Use cases:
        User can set a list of root objects, a destination site, a transfer option set, a list of traverse options and
        a list of session options. All the objects which can be traversed by the option set and options will be
        exported into an output TC XML file. The physical Iman files related exported dataset objects will be
        downloaded and packed into the briefcase file along with the TC XML file.
        """
        return cls.execute_soa_method(
            method_name='exportObjectsToOfflinePackage',
            library='GlobalMultiSite',
            service_date='2008_06',
            service_name='ImportExport',
            params={'reason': reason, 'sites': sites, 'objects': objects, 'optionSetTag': optionSetTag, 'optionNamesAndValues': optionNamesAndValues, 'sessionOptionAndValues': sessionOptionAndValues},
            response_cls=ExportObjectsToOfflinePackageResponse,
        )
